/**
 * Copyright (C) 2012 - present by OpenGamma Inc. and the OpenGamma group of companies
 * 
 * Please see distribution for license.
 */
package com.opengamma.analytics.financial.curve.building; 

import java.util.ArrayList; 
import java.util.Arrays; 
import java.util.LinkedHashMap; 
import java.util.List; 

import org.apache.commons.lang.ArrayUtils; 

import com.opengamma.analytics.financial.curve.generator.GeneratorYDCurve; 
import com.opengamma.analytics.financial.curve.sensitivity.ParameterUnderlyingSensitivityCalculator; 
import com.opengamma.analytics.financial.interestrate.InstrumentDerivative; 
import com.opengamma.analytics.financial.interestrate.InstrumentDerivativeVisitor; 
import com.opengamma.analytics.financial.interestrate.InterestRateCurveSensitivity; 
import com.opengamma.analytics.financial.interestrate.YieldCurveBundle; 
import com.opengamma.analytics.math.function.Function1D; 
import com.opengamma.analytics.math.linearalgebra.DecompositionFactory; 
import com.opengamma.analytics.math.matrix.CommonsMatrixAlgebra; 
import com.opengamma.analytics.math.matrix.DoubleMatrix1D; 
import com.opengamma.analytics.math.matrix.DoubleMatrix2D; 
import com.opengamma.analytics.math.matrix.MatrixAlgebra; 
import com.opengamma.analytics.math.rootfinding.newton.BroydenVectorRootFinder; 
import com.opengamma.util.tuple.ObjectsPair; 
import com.opengamma.util.tuple.Pair; 
import com.opengamma.lambdava.tuple.ObjectsPair; 
import com.opengamma.lambdava.tuple.Pair; 

/**
 * Functions to build curves.
 * TODO: REVIEW: Change the static methods, embed then in an object.
 */
    CurveBuildingFunction {
	

  /**
   * The absolute tolerance for the root finder.
   */
  

	
  /**
   * The relative tolerance for the root finder.
   */
  

	
  /**
   * The relative tolerance for the root finder.
   */
  

	
  /**
   * The root finder used for curve calibration.
   */
  

	
  /**
   * The matrix algebra used for matrix inversion.
   */
  

	

  /**
   * Constructor.
   * @param toleranceAbs The absolute tolerance for the root finder.
   * @param toleranceRel The relative tolerance for the root finder.
   * @param stepMaximum The maximum number of step for the root finder.
   */
  

	

  /**
   * Build a unit of curves.
   * @param instruments The instruments used for the unit calibration.
   * @param initGuess The initial parameters guess.
   * @param curveGenerators The map of curve names to curve generators used to build the unit.
   * @param knownData The known data (fx rates, other curves, model parameters, ...)
   * @param calculator The calculator of the value on which the calibration is done (usually ParSpreadMarketQuoteCalculator (recommended) or converted present value).
   * @param sensitivityCalculator The parameter sensitivity calculator.
   * @return The new curves and the calibrated parameters.
   */
  

	

  /**
   * Build the Jacobian matrixes associated to a unit of curves.
   * @param instruments The instruments used for the block calibration.
   * @param curveGenerators The map of curve names to curve generators used to build the block.
   * @param startBlock The index of the first parameter of the unit in the block.
   * @param nbParameters The number of parameters for each curve in the unit.
   * @param parameters The parameters used to build each curve in the block.
   * @param knownData The known data (FX rates, other curves, model parameters, ...) for the block calibration.
   * @param sensitivityCalculator The parameter sensitivity calculator for the value on which the calibration is done (usually ParSpreadMarketQuoteCalculator (recommended) or converted present value).
   * @return The part of the inverse Jacobian matrix associated to each curve. 
   * The Jacobian matrix is the transition matrix between the curve parameters and the par spread.
   * TODO: Currently only for the ParSpreadMarketQuoteCalculator.
   */
  

	

  /**
   * Build a block of curves.
   * @param instruments The instruments used for the block calibration.
   * @param curveGenerators The curve generators (final version). As an array of arrays, representing the units and the curves within the units.
   * @param curveNames The names of the different curves. As an array of arrays, representing the units and the curves within the units.
   * @param parametersGuess The initial guess for the parameters. As an array of arrays, representing the units and the parameters for one unit (all the curves of the unit concatenated).
   * @param knownData The known data (fx rates, other curves, model parameters, ...)
   * @param calculator The calculator of the value on which the calibration is done (usually ParSpreadMarketQuoteCalculator (recommended) or converted present value).
   * @param sensitivityCalculator The parameter sensitivity calculator.
   * @return A pair with the calibrated yield curve bundle (including the known data) and the CurveBuildingBlckBundle with the relevant inverse Jacobian Matrix.
   */
  


}
