/**
 * Copyright (C) 2012 - present by OpenGamma Inc. and the OpenGamma group of companies
 * 
 * Please see distribution for license.
 */
package com.opengamma.analytics.financial.curve.sensitivity; 

import java.util.Collections; 
import java.util.LinkedHashMap; 
import java.util.Map; 
import java.util.Set; 

import org.apache.commons.lang.ObjectUtils; 

import com.opengamma.analytics.financial.forex.method.FXMatrix; 
import com.opengamma.analytics.math.matrix.DoubleMatrix1D; 
import com.opengamma.analytics.math.matrix.MatrixAlgebra; 
import com.opengamma.analytics.math.matrix.MatrixAlgebraFactory; 
import com.opengamma.util.ArgumentChecker; 
import com.opengamma.util.money.Currency; 
import com.opengamma.util.tuple.Pair; 
import com.opengamma.lambdava.tuple.Pair; 

/**
 * Class containing the sensitivity of the present value to specific parameters or market quotes and methods for manipulating these data. 
 * The vector of sensitivities is stored with reference to a curve name and currency pair, with the sensitivity amounts assumed to be in the currency of the pair. 
 * The order of the sensitivities are important for curve calibration, so the data are stored in such a way as to preserve the order of insertion. 
 */
    ParameterSensitivity {
	
  /**
   * The map containing the sensitivity. The map links a pair of curve name and currency to a vector of sensitivities (sensitivities to parameters/inputs).
   * The sensitivity is expressed in the currency of the pair. 
   */
  

	

  /**
   * Default constructor, creating an empty LinkedHashMap for the sensitivity.
   */
  

	

  /**
   * Constructor taking a map.
   * @param sensitivity The map with the sensitivities, not null. The map is copied. 
   */
  

	

  /**
   * Static constructor from a map. The map is copied.
   * @param sensitivity A map of name / currency pairs to vector of sensitivities, not null 
   * @return An instance of ParameterSensitivity
   */
  

	

  /**
   * Create a copy of the sensitivity and add a given named sensitivity to it. If the name / currency pair is in the map, the two sensitivity matrices are added.
   * Otherwise, a new entry is put into the map
   * @param nameCcy The name and the currency, not null
   * @param sensitivity The sensitivity to add, not null
   * @return The total sensitivity.
   */
  

	

  /**
   * Create a copy of the sensitivity and add a given sensitivity to it.
   * @param other The sensitivity to add.
   * @return The total sensitivity.
   */
  

	

  /**
   * Create a copy of the object with all the sensitivities multiplied by a common factor.
   * @param factor The factor.
   * @return The multiplied sensitivity.
   */
  

	

  /**
   * Create a new parameter sensitivity with the new sensitivity with all the values in a common currency.
   * @param fxMatrix The matrix with relevant exchange rates, not null
   * @param ccy The currency in which the sensitivity is converted, not null
   * @return The converted sensitivity.
   */
  

	

  /**
   * Returns the sensitivities wrapped in an unmodifiable map
   * @return The sensitivities
   */
  

	

  /**
   * Returns the sensitivity for a given name/currency pair.
   * @param nameCcy The name and the currency, not null
   * @return The sensitivity.
   */
  

	

  /**
   * Returns the sensitivity for a given name.
   * @param name The name.
   * @param ccy The currency.
   * @return The sensitivity.
   */
  

	

  /**
   * Returns a set with all the curve names.
   * @return The set of names.
   */
  

	

  

	

  

	

  


}
