/**
 * Copyright (C) 2011 - present by OpenGamma Inc. and the OpenGamma group of companies
 * 
 * Please see distribution for license.
 */
package com.opengamma.analytics.financial.interestrate.market.description; 

import java.util.List; 
import java.util.Set; 

import com.opengamma.analytics.financial.forex.method.FXMatrix; 
import com.opengamma.analytics.financial.instrument.index.IborIndex; 
import com.opengamma.analytics.financial.instrument.index.IndexON; 
import com.opengamma.analytics.financial.instrument.index.IndexPrice; 
import com.opengamma.util.money.Currency; 
import com.opengamma.util.tuple.DoublesPair; 
import com.opengamma.util.tuple.Pair; 
import com.opengamma.lambdava.tuple.DoublesPair; 
import com.opengamma.lambdava.tuple.Pair; 

/**
 * Interface of a "market" providing discounting factors, forward rate (linked to Ibor index), inflation price index and issuer specific curves.
 */
    IMarketBundle {
	

  /**
   * Create a new copy of the bundle.
   * @return The bundle.
   */
  

	

  /**
   * Gets the discount factor for one currency at a given time to maturity.
   * @param ccy The currency.
   * @param time The time.
   * @return The discount factor.
   */
  

	

  /**
   * Return the name associated to the discounting in a currency.
   * @param ccy The currency.
   * @return The name.
   */
  

	

  /**
   * Returns a set of all the currencies available for discounting.
   * @return The currency set.
   */
  

	

  /**
   * Computes the sensitivity to the parameters of the given currency discounting curve from the sensitivity to yield (continuously compounded) at intermediary points.
   * @param ccy The currency.
   * @param pointSensitivity The point yield sensitivity.
   * @return The parameters sensitivity.
   */
  

	

  /**
   * Returns the number of parameters associated to a currency.
   * @param ccy The currency.
   * @return The number of parameters.
   */
  

	

  /**
   * Gets the forward for one Ibor index between start and end times.
   * @param index The Ibor index.
   * @param startTime The start time.
   * @param endTime The end time.
   * @param accrualFactor The Ibor accrual factor.
   * @return The forward rate.
   */
  

	

  /**
   * Gets the forward for one Ibor index between start and end times.
   * @param index The Ibor index.
   * @param startTime The start time.
   * @return The forward rate.
   */
  

	

  /**
   * Returns the curve name associated to an index.
   * @param index The index.
   * @return The name.
   */
  

	

  /**
   * Returns a set of all the index available for forward.
   * @return The index set.
   */
  

	

  /**
   * Computes the sensitivity to the parameters of the given index forward curve from the sensitivity to forward rate at intermediary points.
   * @param index The index.
   * @param pointSensitivity The point yield sensitivity.
   * @return The parameters sensitivity.
   */
  

	

  /**
   * Returns the number of parameters associated to an index.
   * @param index The index.
   * @return The number of parameters.
   */
  

	

  /**
   * Gets the forward for one Ibor index between start and end times.
   * @param index The Ibor index.
   * @param startTime The start time.
   * @param endTime The end time.
   * @param accrualFactor The Ibor accrual factor.
   * @return The forward rate.
   */
  

	

  /**
   * Gets the forward for one Ibor index between start and end times.
   * @param index The Ibor index.
   * @param startTime The start time.
   * @return The forward rate.
   */
  

	

  /**
   * Returns the curve name associated to an index.
   * @param index The index.
   * @return The name.
   */
  

	

  /**
   * Returns a set of all the index available for forward.
   * @return The index set.
   */
  

	

  /**
   * Computes the sensitivity to the parameters of the given index forward curve from the sensitivity to forward rate at intermediary points.
   * @param index The index.
   * @param pointSensitivity The point yield sensitivity.
   * @return The parameters sensitivity.
   */
  

	

  /**
   * Returns the number of parameters associated to an index.
   * @param index The index.
   * @return The number of parameters.
   */
  

	

  /**
   * Gets the estimated price index for a given reference time.
   * @param index The price index.
   * @param time The reference time.
   * @return The price index.
   */
  

	

  /**
   * Return the name associated to a price index.
   * @param index The price index.
   * @return The name.
   */
  

	

  /**
   * Gets the discount factor for one issuer in one currency.
   * @param issuerCcy The issuer name/currency pair.
   * @param time The time.
   * @return The discount factor.
   */
  

	

  /**
   * Gets the set of price indexes defined in the market.
   * @return The set of index.
   */
  

	

  /**
   * Gets the set of issuer names by currency defined in the market.
   * @return The set of issuers names/currencies.
   */
  

	

  /**
   * Gets the names of all curves (discounting, forward, price index and issuers).
   * @return The names.
   */
  

	

  /**
   * Return the exchange rate between two currencies.
   * @param ccy1 The first currency.
   * @param ccy2 The second currency.
   * @return The exchange rate: 1.0 * ccy1 = x * ccy2.
   */
  

	

  /**
   * Gets the underlying FXMatrix containing the exchange rates.
   * @return The matrix.
   */
  


}
