/**
 * Copyright (C) 2011 - present by OpenGamma Inc. and the OpenGamma group of companies
 * 
 * Please see distribution for license.
 */
package com.opengamma.analytics.financial.interestrate.market.description; 

import java.util.LinkedHashMap; 
import java.util.List; 
import java.util.Map; 
import java.util.Set; 
import java.util.TreeSet; 

import com.opengamma.analytics.financial.forex.method.FXMatrix; 
import com.opengamma.analytics.financial.instrument.index.IborIndex; 
import com.opengamma.analytics.financial.instrument.index.IndexON; 
import com.opengamma.analytics.financial.instrument.index.IndexPrice; 
import com.opengamma.analytics.financial.model.interestrate.curve.PriceIndexCurve; 
import com.opengamma.analytics.financial.model.interestrate.curve.YieldAndDiscountCurve; 
import com.opengamma.util.ArgumentChecker; 
import com.opengamma.util.money.Currency; 
import com.opengamma.util.tuple.DoublesPair; 
import com.opengamma.util.tuple.ObjectsPair; 
import com.opengamma.util.tuple.Pair; 
import com.opengamma.lambdava.tuple.DoublesPair; 
import com.opengamma.lambdava.tuple.ObjectsPair; 
import com.opengamma.lambdava.tuple.Pair; 

/**
 * Class describing a "market" with discounting, forward, price index and credit curves.
 * The forward rate are computed as the ratio of discount factors stored in YieldAndDiscountCurve.
 */
    MarketDiscountBundle   {
	

  /**
   * A map with one (discounting) curve by currency.
   */
  

	
  /**
   * A map with one (forward) curve by ON index.
   */
  

	
  /**
   * A map with one (forward) curve by Ibor index.
   */
  

	
  /**
   * A map with one price curve by price index.
   */
  

	
  /**
   * A map with issuer discounting curves.
   */
  

	
  /**
   * The matrix containing the exchange rates.
   */
  

	

  /**
   * Constructor with empty maps for discounting, forward and price index.
   */
  

	

  /**
   * Constructor with empty maps for discounting, forward and price index.
   * @param fxMatrix The FXMatrix.
   */
  

	

  /**
   * Constructor from an existing market. The given market maps are used for the new market (the same maps are used, not copied).
   * @param discountingCurves A map with one (discounting) curve by currency.
   * @param forwardIborCurves A map with one (forward) curve by Ibor index.
   * @param forwardONCurves A map with one (forward) curve by ON index.
   * @param priceIndexCurves A map with one price curve by price index.
   * @param issuerCurves A map with issuer discounting curves.
   * @param fxMatrix The FXMatrix.
   */
  

	

  /**
   * Constructor from an existing market. The given market maps are used for the new market (the same maps are used, not copied).
   * @param discountingCurves A map with one (discounting) curve by currency.
   * @param forwardIborCurves A map with one (forward) curve by Ibor index.
   * @param forwardONCurves A map with one (forward) curve by ON index.
   * @param priceIndexCurves A map with one price curve by price index.
   * @param issuerCurves A map with issuer discounting curves.
   */
  

	

  /**
   * Constructor from exiting maps. The given maps are used for the new market (the same maps are used, not copied).
   * @param market The existing market.
   */
  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  /**
   * Gets the discounting curve associated in a given currency in the market.
   * @param ccy The currency.
   * @return The curve.
   */
  

	

  /**
   * Gets the forward curve associated to a given Ibor index in the market.
   * @param index The Ibor index.
   * @return The curve.
   */
  

	

  /**
   * Gets the forward curve associated to a given ON index in the market.
   * @param index The ON index.
   * @return The curve.
   */
  

	

  /**
   * Gets the price index curve associated to a given price index in the market.
   * @param index The Price index.
   * @return The curve.
   */
  

	

  /**
   * Gets the discounting curve associated to an issuer.
   * @param issuerCcy The issuer name/currency pair.
   * @return The curve.
   */
  

	

  

	

  

	

  

	

  /**
   * Sets the discounting curve for a given currency.
   * @param ccy The currency.
   * @param curve The yield curve used for discounting.
   */
  

	

  /**
   * Sets the curve associated to an Ibor index.
   * @param index The index.
   * @param curve The curve.
   */
  

	

  /**
   * Sets the curve associated to an ON index.
   * @param index The index.
   * @param curve The curve.
   */
  

	

  /**
   * Sets the price index curve for a price index.
   * @param index The price index.
   * @param curve The curve.
   */
  

	

  /**
   * Sets the curve associated to an issuer.
   * @param issuer The issuer name.
   * @param ccy The currency.
   * @param curve The curve.
   */
  

	

  /**
   * Set all the curves contains in another bundle. If a currency or index is already present in the map, the associated curve is changed.
   * @param other The other bundle.
   * TODO: REVIEW: Should we check that the curve are already present?
   */
  

	

  /**
   * Replaces the discounting curve for a given currency.
   * @param ccy The currency.
   * @param curve The yield curve used for discounting.
   *  @throws IllegalArgumentException if curve name NOT already present 
   */
  

	

  /**
   * Replaces the forward curve for a given index.
   * @param index The index.
   * @param curve The yield curve used for forward.
   *  @throws IllegalArgumentException if curve name NOT already present 
   */
  

	

  /**
   * Replaces the discounting curve for a price index.
   * @param index The price index.
   * @param curve The price curve for the index.
   *  @throws IllegalArgumentException if curve name NOT already present 
   */
  

	

  

	

  /**
   * Gets the underlying FXMatrix containing the exchange rates.
   * @return The matrix.
   */
  


}
