/**
 * Copyright (C) 2012 - present by OpenGamma Inc. and the OpenGamma group of companies
 * 
 * Please see distribution for license.
 */
package com.opengamma.analytics.financial.curve; 

import static org.testng.AssertJUnit.assertEquals; 
import static org.threeten.bp.temporal.ChronoUnit.DAYS; 
import static org.threeten.bp.temporal.ChronoUnit.MONTHS; 
import static org.threeten.bp.temporal.ChronoUnit.YEARS; 

import java.util.ArrayList; 
import java.util.HashMap; 
import java.util.List; 
import java.util.Set; 

import org.testng.annotations.BeforeSuite; 
import org.testng.annotations.Test; 
import org.threeten.bp.Period; 
import org.threeten.bp.ZonedDateTime; 

import com.opengamma.analytics.financial.calculator.PresentValueMCACalculator; 
import com.opengamma.analytics.financial.curve.building.CurveBuildingBlockBundle; 
import com.opengamma.analytics.financial.curve.building.CurveBuildingFunction; 
import com.opengamma.analytics.financial.curve.generator.GeneratorCurveYieldInterpolated; 
import com.opengamma.analytics.financial.curve.generator.GeneratorYDCurve; 
import com.opengamma.analytics.financial.forex.method.FXMatrix; 
import com.opengamma.analytics.financial.instrument.InstrumentDefinition; 
import com.opengamma.analytics.financial.instrument.bond.BillSecurityDefinition; 
import com.opengamma.analytics.financial.instrument.bond.BillTransactionDefinition; 
import com.opengamma.analytics.financial.instrument.cash.CashDefinition; 
import com.opengamma.analytics.financial.instrument.cash.DepositCounterpartDefinition; 
import com.opengamma.analytics.financial.instrument.index.GeneratorBill; 
import com.opengamma.analytics.financial.instrument.index.GeneratorDepositON; 
import com.opengamma.analytics.financial.instrument.index.GeneratorDepositONCounterpart; 
import com.opengamma.analytics.financial.instrument.index.GeneratorInstrument; 
import com.opengamma.analytics.financial.instrument.index.GeneratorSwapFixedON; 
import com.opengamma.analytics.financial.instrument.index.IndexON; 
import com.opengamma.analytics.financial.instrument.swap.SwapFixedONDefinition; 
import com.opengamma.analytics.financial.interestrate.InstrumentDerivative; 
import com.opengamma.analytics.financial.interestrate.InstrumentDerivativeVisitorAdapter; 
import com.opengamma.analytics.financial.interestrate.InterestRateCurveSensitivity; 
import com.opengamma.analytics.financial.interestrate.LastTimeCalculator; 
import com.opengamma.analytics.financial.interestrate.ParSpreadMarketQuoteCalculator; 
import com.opengamma.analytics.financial.interestrate.ParSpreadMarketQuoteCurveSensitivityCalculator; 
import com.opengamma.analytics.financial.interestrate.YieldCurveBundle; 
import com.opengamma.analytics.math.interpolation.CombinedInterpolatorExtrapolatorFactory; 
import com.opengamma.analytics.math.interpolation.Interpolator1D; 
import com.opengamma.analytics.math.interpolation.Interpolator1DFactory; 
import com.opengamma.financial.convention.businessday.BusinessDayConvention; 
import com.opengamma.financial.convention.businessday.BusinessDayConventionFactory; 
import com.opengamma.financial.convention.calendar.Calendar; 
import com.opengamma.financial.convention.calendar.MondayToFridayCalendar; 
import com.opengamma.financial.convention.daycount.DayCount; 
import com.opengamma.financial.convention.daycount.DayCountFactory; 
import com.opengamma.financial.convention.yield.YieldConvention; 
import com.opengamma.financial.convention.yield.YieldConventionFactory; 
import com.opengamma.util.money.Currency; 
import com.opengamma.util.time.DateUtils; 
import com.opengamma.util.timeseries.DoubleTimeSeries; 
import com.opengamma.util.timeseries.zoneddatetime.ArrayZonedDateTimeDoubleTimeSeries; 
import com.opengamma.lambdava.tuple.Pair; 

/**
 * Build curves using bills and bonds.
 */
public  class  CurveConstructionBillBondTest {
	

  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	

  

	
  

	

  

	
  

	
  

	
  

	
  

	
  

	
  

	

  

	

  

	
  

	
  

	
  

	
  

	

  

	
  

	
  

	

  /** Market values for the dsc USD curve */
  

	
  /** Generators for the dsc USD curve */
  

	
  /** Tenors for the dsc USD curve */
  

	

  /** Market values for the govt USD curve */
  

	
  /** Generators for the govt USD curve */
  

	
  /** Tenors for the govt USD curve */
  

	

  /** Standard USD discounting curve instrument definitions */
  

	
  /** Standard USD Forward 3M curve instrument definitions */
  

	
  /** Units of curves */
  

	
  

	
  

	
  

	
  

	
  

	

  // Calculators
  

	
  

	
  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	
  static {
    for (int loopbill = 0; loopbill < BILL_MATURITY.length; loopbill++) {
      BILL_SECURITY[loopbill] = new BillSecurityDefinition(CCY_USD, BILL_MATURITY[loopbill], NOTIONAL, SPOT_LAG_BILL, CALENDAR, YIELD_BILL_USGOVT, DAY_COUNT_BILL_USGOVT, NAME_COUNTERPART);
      GENERATOR_BILL[loopbill] = new GeneratorBill("GeneratorBill" + loopbill, BILL_SECURITY[loopbill]);
    }
  }

	
  static {
    CCY_MAP.put(CURVE_NAME_DSC_USD, CCY_USD);
    CCY_MAP.put(CURVE_NAME_GOVTUS_USD, CCY_USD);
  }

	

  static {
    DEFINITIONS_DSC_USD = getDefinitions(DSC_USD_MARKET_QUOTES, DSC_USD_GENERATORS, DSC_USD_TENOR);
    DEFINITIONS_GOVTUS_USD = getDefinitions(GOVTUS_USD_MARKET_QUOTES, GOVTUS_USD_GENERATORS, GOVTUS_USD_TENOR);
    for (int loopblock = 0; loopblock < NB_BLOCKS; loopblock++) {
      DEFINITIONS_UNITS[loopblock] = new InstrumentDefinition<?>[NB_UNITS[loopblock]][][];
      GENERATORS_UNITS[loopblock] = new GeneratorYDCurve[NB_UNITS[loopblock]][];
      NAMES_UNITS[loopblock] = new String[NB_UNITS[loopblock]][];
    }
    DEFINITIONS_UNITS[0][0] = new InstrumentDefinition<?>[][] {DEFINITIONS_DSC_USD };
    DEFINITIONS_UNITS[0][1] = new InstrumentDefinition<?>[][] {DEFINITIONS_GOVTUS_USD };
    final GeneratorYDCurve genIntLin = new GeneratorCurveYieldInterpolated(MATURITY_CALCULATOR, INTERPOLATOR_LINEAR);
    GENERATORS_UNITS[0][0] = new GeneratorYDCurve[] {genIntLin };
    GENERATORS_UNITS[0][1] = new GeneratorYDCurve[] {genIntLin };
    NAMES_UNITS[0][0] = new String[] {CURVE_NAME_DSC_USD };
    NAMES_UNITS[0][1] = new String[] {CURVE_NAME_GOVTUS_USD };
  }


}
