/**
 * Copyright (C) 2012 - present by OpenGamma Inc. and the OpenGamma group of companies
 * 
 * Please see distribution for license.
 */
package com.opengamma.analytics.financial.curve; 

import static org.threeten.bp.temporal.ChronoUnit.DAYS; 
import static org.threeten.bp.temporal.ChronoUnit.MONTHS; 
import static org.threeten.bp.temporal.ChronoUnit.YEARS; 

import java.util.ArrayList; 
import java.util.HashMap; 
import java.util.LinkedHashMap; 
import java.util.List; 

import org.threeten.bp.Period; 
import org.threeten.bp.ZonedDateTime; 

import com.opengamma.analytics.financial.curve.building.MultipleYieldCurveFinderGeneratorDataBundle; 
import com.opengamma.analytics.financial.curve.building.MultipleYieldCurveFinderGeneratorFunction; 
import com.opengamma.analytics.financial.curve.building.MultipleYieldCurveFinderGeneratorJacobian; 
import com.opengamma.analytics.financial.curve.generator.GeneratorCurveYieldInterpolatedNode; 
import com.opengamma.analytics.financial.curve.generator.GeneratorYDCurve; 
import com.opengamma.analytics.financial.curve.sensitivity.ParameterSensitivityCalculator; 
import com.opengamma.analytics.financial.forex.method.FXMatrix; 
import com.opengamma.analytics.financial.instrument.InstrumentDefinition; 
import com.opengamma.analytics.financial.instrument.cash.CashDefinition; 
import com.opengamma.analytics.financial.instrument.fra.ForwardRateAgreementDefinition; 
import com.opengamma.analytics.financial.instrument.index.GeneratorDeposit; 
import com.opengamma.analytics.financial.instrument.index.GeneratorDepositON; 
import com.opengamma.analytics.financial.instrument.index.GeneratorForexSwap; 
import com.opengamma.analytics.financial.instrument.index.GeneratorInstrument; 
import com.opengamma.analytics.financial.instrument.index.GeneratorSwapFixedIbor; 
import com.opengamma.analytics.financial.instrument.index.GeneratorSwapFixedIborMaster; 
import com.opengamma.analytics.financial.instrument.index.GeneratorSwapFixedON; 
import com.opengamma.analytics.financial.instrument.index.GeneratorSwapXCcyIborIbor; 
import com.opengamma.analytics.financial.instrument.index.IborIndex; 
import com.opengamma.analytics.financial.instrument.index.IndexON; 
import com.opengamma.analytics.financial.instrument.swap.SwapFixedIborDefinition; 
import com.opengamma.analytics.financial.instrument.swap.SwapFixedONDefinition; 
import com.opengamma.analytics.financial.instrument.swap.SwapXCcyIborIborDefinition; 
import com.opengamma.analytics.financial.interestrate.InstrumentDerivative; 
import com.opengamma.analytics.financial.interestrate.InstrumentDerivativeVisitor; 
import com.opengamma.analytics.financial.interestrate.InstrumentDerivativeVisitorAdapter; 
import com.opengamma.analytics.financial.interestrate.InterestRateCurveSensitivity; 
import com.opengamma.analytics.financial.interestrate.LastTimeCalculator; 
import com.opengamma.analytics.financial.interestrate.PresentValueBasisPointCalculator; 
import com.opengamma.analytics.financial.interestrate.YieldCurveBundle; 
import com.opengamma.analytics.math.function.Function1D; 
import com.opengamma.analytics.math.interpolation.CombinedInterpolatorExtrapolatorFactory; 
import com.opengamma.analytics.math.interpolation.Interpolator1D; 
import com.opengamma.analytics.math.interpolation.Interpolator1DFactory; 
import com.opengamma.analytics.math.linearalgebra.DecompositionFactory; 
import com.opengamma.analytics.math.matrix.CommonsMatrixAlgebra; 
import com.opengamma.analytics.math.matrix.DoubleMatrix1D; 
import com.opengamma.analytics.math.matrix.DoubleMatrix2D; 
import com.opengamma.analytics.math.matrix.MatrixAlgebra; 
import com.opengamma.analytics.math.rootfinding.newton.BroydenVectorRootFinder; 
import com.opengamma.financial.convention.businessday.BusinessDayConvention; 
import com.opengamma.financial.convention.businessday.BusinessDayConventionFactory; 
import com.opengamma.financial.convention.calendar.Calendar; 
import com.opengamma.financial.convention.calendar.MondayToFridayCalendar; 
import com.opengamma.financial.convention.daycount.DayCount; 
import com.opengamma.financial.convention.daycount.DayCountFactory; 
import com.opengamma.util.money.Currency; 
import com.opengamma.util.time.DateUtils; 
import com.opengamma.util.timeseries.DoubleTimeSeries; 
import com.opengamma.util.timeseries.zoneddatetime.ArrayZonedDateTimeDoubleTimeSeries; 
import com.opengamma.lambdava.tuple.ObjectsPair; 
import com.opengamma.lambdava.tuple.Pair; 

/**
 * Data for the testing of curve construction with FX swaps and XCCy swaps
 */
public  class  YieldCurveConstructionGeneratorTwoCurrenciesXCcyData {
	

  

	
  

	

  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	 // Spread on EUR leg

  

	

  

	
  

	
  

	
  

	
  

	

  

	
  

	

  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	

  

	
  

	
  

	
  

	
  

	

  /** Market values for the dsc USD curve */
  

	
  /** Generators for the dsc USD curve */
  

	
  /** Tenors for the dsc USD curve */
  

	

  /** Market values for the Fwd 3M USD curve */
  

	
  /** Generators for the Fwd 3M USD curve */
  

	
  /** Tenors for the Fwd 3M USD curve */
  

	

  /** Market values for the dsc EUR curve */
  

	
  /** Generators for the dsc EUR curve */
  

	
  /** Tenors for the dsc EUR curve */
  

	
  //  /** FX rqtes for the dsc EUR curve */
  //  public static final Double[] DSC_2_FX_RATE = new Double[] {FX_EURUSD, FX_EURUSD, FX_EURUSD, FX_EURUSD, FX_EURUSD, FX_EURUSD, FX_EURUSD, FX_EURUSD, FX_EURUSD, FX_EURUSD, FX_EURUSD, FX_EURUSD,
  //      FX_EURUSD};

  /** Market values for the Fwd 3M EUR curve */
  

	
  /** Generators for the Fwd 3M USD curve */
  

	
  /** Tenors for the Fwd 3M USD curve */
  

	

  /** Standard USD discounting curve instrument definitions */
  

	
  /** Standard USD Forward 3M curve instrument definitions */
  

	
  /** Standard EUR discounting curve instrument definitions */
  

	
  /** Standard EUR Forward 3M curve instrument definitions */
  

	

  /**
   * Curve types used in the curve construction
   */
   enum  CurveType {
    /** USD Discounting **/
     , 
    /** EUR Discounting **/
     , 
    /** USD Forward 3M **/
     , 
    /** EUR Forward 3M **/
    ;}

	

  

	

  

	

  

	

  

	

  static {
    DEFINITIONS_DSC_1 = getDefinitions(DSC_1_MARKET_QUOTES, DSC_1_GENERATORS, DSC_1_TENOR);
    DEFINITIONS_FWD_1 = getDefinitions(FWD_1_MARKET_QUOTES, FWD_1_GENERATORS, FWD_1_TENOR);
    DEFINITIONS_DSC_2 = getDefinitions(DSC_2_MARKET_QUOTES, DSC_2_GENERATORS, DSC_2_TENOR);
    DEFINITIONS_FWD_2 = getDefinitions(FWD_2_MARKET_QUOTES, FWD_2_GENERATORS, FWD_2_TENOR);
    CCY_MAP.put(CURVE_NAME_DSC_1, CCY1);
    CCY_MAP.put(CURVE_NAME_FWD_1, CCY1);
    CCY_MAP.put(CURVE_NAME_DSC_2, CCY2);
    CCY_MAP.put(CURVE_NAME_FWD_2, CCY2);
  }


}
