/**
 * Copyright (C) 2011 - present by OpenGamma Inc. and the OpenGamma group of companies
 * 
 * Please see distribution for license.
 */
package com.opengamma.analytics.financial.interestrate; 

import static com.opengamma.analytics.math.interpolation.Interpolator1DFactory.FLAT_EXTRAPOLATOR; 
import static com.opengamma.analytics.math.interpolation.Interpolator1DFactory.LINEAR_EXTRAPOLATOR; 
import static org.testng.AssertJUnit.assertEquals; 
import static org.threeten.bp.temporal.ChronoUnit.MONTHS; 

import java.util.ArrayList; 
import java.util.Arrays; 
import java.util.HashMap; 
import java.util.LinkedHashMap; 
import java.util.List; 
import java.util.Map; 

import org.slf4j.Logger; 
import org.slf4j.LoggerFactory; 
import org.testng.annotations.Test; 
import org.threeten.bp.Period; 
import org.threeten.bp.ZonedDateTime; 

import com.opengamma.analytics.financial.instrument.annuity.AnnuityCouponFixedDefinition; 
import com.opengamma.analytics.financial.instrument.annuity.AnnuityCouponIborDefinition; 
import com.opengamma.analytics.financial.instrument.cash.CashDefinition; 
import com.opengamma.analytics.financial.instrument.fra.ForwardRateAgreementDefinition; 
import com.opengamma.analytics.financial.instrument.index.IborIndex; 
import com.opengamma.analytics.financial.instrument.swap.SwapFixedIborDefinition; 
import com.opengamma.analytics.financial.interestrate.cash.derivative.Cash; 
import com.opengamma.analytics.financial.interestrate.payments.derivative.Payment; 
import com.opengamma.analytics.financial.interestrate.swap.derivative.Swap; 
import com.opengamma.analytics.financial.model.interestrate.curve.YieldAndDiscountCurve; 
import com.opengamma.analytics.financial.schedule.ScheduleCalculator; 
import com.opengamma.analytics.math.function.Function1D; 
import com.opengamma.analytics.math.interpolation.CombinedInterpolatorExtrapolator; 
import com.opengamma.analytics.math.interpolation.CombinedInterpolatorExtrapolatorFactory; 
import com.opengamma.analytics.math.interpolation.Interpolator1DFactory; 
import com.opengamma.analytics.math.linearalgebra.DecompositionFactory; 
import com.opengamma.analytics.math.matrix.DoubleMatrix1D; 
import com.opengamma.analytics.math.matrix.DoubleMatrix2D; 
import com.opengamma.analytics.math.rootfinding.YieldCurveFittingSetup; 
import com.opengamma.analytics.math.rootfinding.YieldCurveFittingTestDataBundle; 
import com.opengamma.analytics.math.rootfinding.newton.BroydenVectorRootFinder; 
import com.opengamma.analytics.math.rootfinding.newton.NewtonVectorRootFinder; 
import com.opengamma.financial.convention.businessday.BusinessDayConvention; 
import com.opengamma.financial.convention.businessday.BusinessDayConventionFactory; 
import com.opengamma.financial.convention.calendar.MondayToFridayCalendar; 
import com.opengamma.financial.convention.daycount.DayCount; 
import com.opengamma.financial.convention.daycount.DayCountFactory; 
import com.opengamma.financial.convention.frequency.SimpleFrequency; 
import com.opengamma.util.money.Currency; 
import com.opengamma.util.time.DateUtils; 
import com.opengamma.util.timeseries.DoubleTimeSeries; 
import com.opengamma.util.timeseries.zoneddatetime.ArrayZonedDateTimeDoubleTimeSeries; 
import com.opengamma.util.tuple.DoublesPair; 
import com.opengamma.lambdava.tuple.DoublesPair; 

/**
 * 
 */
    InstrumentDefinitionYieldCurveSensitivitiesTest  extends YieldCurveFittingSetup {
	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	

  static {
    SINGLE_CURVE_PAR_RATE_JACOBIAN_FUNCTION = new MultipleYieldCurveFinderJacobian(SINGLE_CURVE_PAR_RATE_DATA,
        SINGLE_CURVE_PAR_RATE_DATA.getMarketValueSensitivityCalculator());
    SINGLE_CURVE_PAR_RATE_YIELD_CURVE_NODES = ROOT_FINDER.getRoot(SINGLE_CURVE_PAR_RATE_FUNCTION, SINGLE_CURVE_PAR_RATE_JACOBIAN_FUNCTION,
        SINGLE_CURVE_PAR_RATE_DATA.getStartPosition());
    SINGLE_CURVE_PAR_RATE_JACOBIAN = SINGLE_CURVE_PAR_RATE_JACOBIAN_FUNCTION.evaluate(SINGLE_CURVE_PAR_RATE_YIELD_CURVE_NODES);
    SINGLE_CURVE_PAR_RATE_CURVES = getYieldCurveMap(SINGLE_CURVE_PAR_RATE_DATA, SINGLE_CURVE_PAR_RATE_YIELD_CURVE_NODES);
    SINGLE_CURVE_PAR_RATE_ALL_CURVES = getAllCurves(SINGLE_CURVE_PAR_RATE_DATA, SINGLE_CURVE_PAR_RATE_CURVES);
    SINGLE_CURVE_PV_JACOBIAN_FUNCTION = new MultipleYieldCurveFinderJacobian(SINGLE_CURVE_PV_DATA, SINGLE_CURVE_PV_DATA.getMarketValueSensitivityCalculator());
    SINGLE_CURVE_PV_YIELD_CURVE_NODES = ROOT_FINDER.getRoot(SINGLE_CURVE_PV_FUNCTION, SINGLE_CURVE_PV_JACOBIAN_FUNCTION, SINGLE_CURVE_PV_DATA.getStartPosition());
    SINGLE_CURVE_PV_JACOBIAN = SINGLE_CURVE_PV_JACOBIAN_FUNCTION.evaluate(SINGLE_CURVE_PV_YIELD_CURVE_NODES);
    SINGLE_CURVE_PV_CURVES = getYieldCurveMap(SINGLE_CURVE_PV_DATA, SINGLE_CURVE_PV_YIELD_CURVE_NODES);
    SINGLE_CURVE_PV_ALL_CURVES = getAllCurves(SINGLE_CURVE_PV_DATA, SINGLE_CURVE_PV_CURVES);
    double[] couponSensitivityArray = new double[SINGLE_CURVE_PV_DATA.getNumInstruments()];
    for (int i = 0; i < SINGLE_CURVE_PV_DATA.getNumInstruments(); i++) {
      couponSensitivityArray[i] = SINGLE_CURVE_PV_DATA.getDerivative(i).accept(PVCS, SINGLE_CURVE_PV_ALL_CURVES);
    }
    SINGLE_CURVE_PV_COUPON_SENSITIVITY = new DoubleMatrix1D(couponSensitivityArray);
    initDoubleCurveMarketCurveData(DOUBLE_CURVE_MARKET_RATES, DOUBLE_CURVE_IRD, DOUBLE_CURVE_NODES);
    DOUBLE_CURVE_PAR_RATE_DATA = getDoubleCurveSetup(ParRateCalculator.getInstance(), ParRateCurveSensitivityCalculator.getInstance(), DOUBLE_CURVE_MARKET_RATES,
        DOUBLE_CURVE_IRD, DOUBLE_CURVE_NODES, false);
    DOUBLE_CURVE_PAR_RATE_FUNCTION = new MultipleYieldCurveFinderFunction(DOUBLE_CURVE_PAR_RATE_DATA, DOUBLE_CURVE_PAR_RATE_DATA.getMarketValueCalculator());
    DOUBLE_CURVE_PAR_RATE_JACOBIAN_FUNCTION = new MultipleYieldCurveFinderJacobian(DOUBLE_CURVE_PAR_RATE_DATA,
        DOUBLE_CURVE_PAR_RATE_DATA.getMarketValueSensitivityCalculator());
    DOUBLE_CURVE_PAR_RATE_YIELD_CURVE_NODES = ROOT_FINDER.getRoot(DOUBLE_CURVE_PAR_RATE_FUNCTION, DOUBLE_CURVE_PAR_RATE_JACOBIAN_FUNCTION,
        DOUBLE_CURVE_PAR_RATE_DATA.getStartPosition());
    DOUBLE_CURVE_PAR_RATE_JACOBIAN = DOUBLE_CURVE_PAR_RATE_JACOBIAN_FUNCTION.evaluate(DOUBLE_CURVE_PAR_RATE_YIELD_CURVE_NODES);
    DOUBLE_CURVE_PAR_RATE_CURVES = getYieldCurveMap(DOUBLE_CURVE_PAR_RATE_DATA, DOUBLE_CURVE_PAR_RATE_YIELD_CURVE_NODES);
    DOUBLE_CURVE_PAR_RATE_ALL_CURVES = getAllCurves(DOUBLE_CURVE_PAR_RATE_DATA, DOUBLE_CURVE_PAR_RATE_CURVES);
    DOUBLE_CURVE_PV_DATA = getDoubleCurveSetup(PresentValueCalculator.getInstance(), PresentValueCurveSensitivityCalculator.getInstance(), DOUBLE_CURVE_MARKET_RATES,
        DOUBLE_CURVE_IRD, DOUBLE_CURVE_NODES, true);
    DOUBLE_CURVE_PV_FUNCTION = new MultipleYieldCurveFinderFunction(DOUBLE_CURVE_PV_DATA, DOUBLE_CURVE_PV_DATA.getMarketValueCalculator());
    DOUBLE_CURVE_PV_JACOBIAN_FUNCTION = new MultipleYieldCurveFinderJacobian(DOUBLE_CURVE_PV_DATA, DOUBLE_CURVE_PV_DATA.getMarketValueSensitivityCalculator());
    DOUBLE_CURVE_PV_YIELD_CURVE_NODES = ROOT_FINDER.getRoot(DOUBLE_CURVE_PV_FUNCTION, DOUBLE_CURVE_PV_JACOBIAN_FUNCTION, DOUBLE_CURVE_PV_DATA.getStartPosition());
    DOUBLE_CURVE_PV_JACOBIAN = DOUBLE_CURVE_PV_JACOBIAN_FUNCTION.evaluate(DOUBLE_CURVE_PV_YIELD_CURVE_NODES);
    DOUBLE_CURVE_PV_CURVES = getYieldCurveMap(DOUBLE_CURVE_PV_DATA, DOUBLE_CURVE_PV_YIELD_CURVE_NODES);
    DOUBLE_CURVE_PV_ALL_CURVES = getAllCurves(DOUBLE_CURVE_PV_DATA, DOUBLE_CURVE_PV_CURVES);
    couponSensitivityArray = new double[DOUBLE_CURVE_PV_DATA.getNumInstruments()];
    for (int i = 0; i < DOUBLE_CURVE_PV_DATA.getNumInstruments(); i++) {
      couponSensitivityArray[i] = DOUBLE_CURVE_PV_DATA.getDerivative(i).accept(PVCS, DOUBLE_CURVE_PV_ALL_CURVES);
    }
    DOUBLE_CURVE_PV_COUPON_SENSITIVITY = new DoubleMatrix1D(couponSensitivityArray);
  }

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

      DummyCurve1  extends Function1D<Double, Double> {
		
    

		
    

		
    

		
    

		
    

		

    


	}

	

      DummySpreadCurve2  extends Function1D<Double, Double> {
		
    

		
    

		
    

		

    


	}

	

  static {
    SINGLE_CURVE_PAR_RATE_JACOBIAN_FUNCTION = new MultipleYieldCurveFinderJacobian(SINGLE_CURVE_PAR_RATE_DATA,
        SINGLE_CURVE_PAR_RATE_DATA.getMarketValueSensitivityCalculator());
    SINGLE_CURVE_PAR_RATE_YIELD_CURVE_NODES = ROOT_FINDER.getRoot(SINGLE_CURVE_PAR_RATE_FUNCTION, SINGLE_CURVE_PAR_RATE_JACOBIAN_FUNCTION,
        SINGLE_CURVE_PAR_RATE_DATA.getStartPosition());
    SINGLE_CURVE_PAR_RATE_JACOBIAN = SINGLE_CURVE_PAR_RATE_JACOBIAN_FUNCTION.evaluate(SINGLE_CURVE_PAR_RATE_YIELD_CURVE_NODES);
    SINGLE_CURVE_PAR_RATE_CURVES = getYieldCurveMap(SINGLE_CURVE_PAR_RATE_DATA, SINGLE_CURVE_PAR_RATE_YIELD_CURVE_NODES);
    SINGLE_CURVE_PAR_RATE_ALL_CURVES = getAllCurves(SINGLE_CURVE_PAR_RATE_DATA, SINGLE_CURVE_PAR_RATE_CURVES);
    SINGLE_CURVE_PV_JACOBIAN_FUNCTION = new MultipleYieldCurveFinderJacobian(SINGLE_CURVE_PV_DATA, SINGLE_CURVE_PV_DATA.getMarketValueSensitivityCalculator());
    SINGLE_CURVE_PV_YIELD_CURVE_NODES = ROOT_FINDER.getRoot(SINGLE_CURVE_PV_FUNCTION, SINGLE_CURVE_PV_JACOBIAN_FUNCTION, SINGLE_CURVE_PV_DATA.getStartPosition());
    SINGLE_CURVE_PV_JACOBIAN = SINGLE_CURVE_PV_JACOBIAN_FUNCTION.evaluate(SINGLE_CURVE_PV_YIELD_CURVE_NODES);
    SINGLE_CURVE_PV_CURVES = getYieldCurveMap(SINGLE_CURVE_PV_DATA, SINGLE_CURVE_PV_YIELD_CURVE_NODES);
    SINGLE_CURVE_PV_ALL_CURVES = getAllCurves(SINGLE_CURVE_PV_DATA, SINGLE_CURVE_PV_CURVES);
    double[] couponSensitivityArray = new double[SINGLE_CURVE_PV_DATA.getNumInstruments()];
    for (int i = 0; i < SINGLE_CURVE_PV_DATA.getNumInstruments(); i++) {
      couponSensitivityArray[i] = SINGLE_CURVE_PV_DATA.getDerivative(i).accept(PVCS, SINGLE_CURVE_PV_ALL_CURVES);
    }
    SINGLE_CURVE_PV_COUPON_SENSITIVITY = new DoubleMatrix1D(couponSensitivityArray);
    initDoubleCurveMarketCurveData(DOUBLE_CURVE_MARKET_RATES, DOUBLE_CURVE_IRD, DOUBLE_CURVE_NODES);
    DOUBLE_CURVE_PAR_RATE_DATA = getDoubleCurveSetup(ParRateCalculator.getInstance(), ParRateCurveSensitivityCalculator.getInstance(), DOUBLE_CURVE_MARKET_RATES,
        DOUBLE_CURVE_IRD, DOUBLE_CURVE_NODES, false);
    DOUBLE_CURVE_PAR_RATE_FUNCTION = new MultipleYieldCurveFinderFunction(DOUBLE_CURVE_PAR_RATE_DATA, DOUBLE_CURVE_PAR_RATE_DATA.getMarketValueCalculator());
    DOUBLE_CURVE_PAR_RATE_JACOBIAN_FUNCTION = new MultipleYieldCurveFinderJacobian(DOUBLE_CURVE_PAR_RATE_DATA,
        DOUBLE_CURVE_PAR_RATE_DATA.getMarketValueSensitivityCalculator());
    DOUBLE_CURVE_PAR_RATE_YIELD_CURVE_NODES = ROOT_FINDER.getRoot(DOUBLE_CURVE_PAR_RATE_FUNCTION, DOUBLE_CURVE_PAR_RATE_JACOBIAN_FUNCTION,
        DOUBLE_CURVE_PAR_RATE_DATA.getStartPosition());
    DOUBLE_CURVE_PAR_RATE_JACOBIAN = DOUBLE_CURVE_PAR_RATE_JACOBIAN_FUNCTION.evaluate(DOUBLE_CURVE_PAR_RATE_YIELD_CURVE_NODES);
    DOUBLE_CURVE_PAR_RATE_CURVES = getYieldCurveMap(DOUBLE_CURVE_PAR_RATE_DATA, DOUBLE_CURVE_PAR_RATE_YIELD_CURVE_NODES);
    DOUBLE_CURVE_PAR_RATE_ALL_CURVES = getAllCurves(DOUBLE_CURVE_PAR_RATE_DATA, DOUBLE_CURVE_PAR_RATE_CURVES);
    DOUBLE_CURVE_PV_DATA = getDoubleCurveSetup(PresentValueCalculator.getInstance(), PresentValueCurveSensitivityCalculator.getInstance(), DOUBLE_CURVE_MARKET_RATES,
        DOUBLE_CURVE_IRD, DOUBLE_CURVE_NODES, true);
    DOUBLE_CURVE_PV_FUNCTION = new MultipleYieldCurveFinderFunction(DOUBLE_CURVE_PV_DATA, DOUBLE_CURVE_PV_DATA.getMarketValueCalculator());
    DOUBLE_CURVE_PV_JACOBIAN_FUNCTION = new MultipleYieldCurveFinderJacobian(DOUBLE_CURVE_PV_DATA, DOUBLE_CURVE_PV_DATA.getMarketValueSensitivityCalculator());
    DOUBLE_CURVE_PV_YIELD_CURVE_NODES = ROOT_FINDER.getRoot(DOUBLE_CURVE_PV_FUNCTION, DOUBLE_CURVE_PV_JACOBIAN_FUNCTION, DOUBLE_CURVE_PV_DATA.getStartPosition());
    DOUBLE_CURVE_PV_JACOBIAN = DOUBLE_CURVE_PV_JACOBIAN_FUNCTION.evaluate(DOUBLE_CURVE_PV_YIELD_CURVE_NODES);
    DOUBLE_CURVE_PV_CURVES = getYieldCurveMap(DOUBLE_CURVE_PV_DATA, DOUBLE_CURVE_PV_YIELD_CURVE_NODES);
    DOUBLE_CURVE_PV_ALL_CURVES = getAllCurves(DOUBLE_CURVE_PV_DATA, DOUBLE_CURVE_PV_CURVES);
    couponSensitivityArray = new double[DOUBLE_CURVE_PV_DATA.getNumInstruments()];
    for (int i = 0; i < DOUBLE_CURVE_PV_DATA.getNumInstruments(); i++) {
      couponSensitivityArray[i] = DOUBLE_CURVE_PV_DATA.getDerivative(i).accept(PVCS, DOUBLE_CURVE_PV_ALL_CURVES);
    }
    DOUBLE_CURVE_PV_COUPON_SENSITIVITY = new DoubleMatrix1D(couponSensitivityArray);
  }


}
