/**
 * Copyright (C) 2009 - present by OpenGamma Inc. and the OpenGamma group of
 * companies
 * 
 * Please see distribution for license.
 */
package com.opengamma.analytics.financial.interestrate; 

import static com.opengamma.analytics.math.interpolation.Interpolator1DFactory.FLAT_EXTRAPOLATOR; 
import static com.opengamma.analytics.math.interpolation.Interpolator1DFactory.LINEAR_EXTRAPOLATOR; 
import static org.testng.AssertJUnit.assertEquals; 

import java.util.ArrayList; 
import java.util.Arrays; 
import java.util.HashMap; 
import java.util.LinkedHashMap; 
import java.util.List; 
import java.util.Map; 

import org.apache.commons.lang.Validate; 
import org.slf4j.Logger; 
import org.slf4j.LoggerFactory; 
import org.testng.annotations.Test; 

import com.opengamma.analytics.financial.model.interestrate.curve.YieldAndDiscountCurve; 
import com.opengamma.analytics.math.differentiation.VectorFieldFirstOrderDifferentiator; 
import com.opengamma.analytics.math.function.Function1D; 
import com.opengamma.analytics.math.interpolation.CombinedInterpolatorExtrapolator; 
import com.opengamma.analytics.math.interpolation.CombinedInterpolatorExtrapolatorFactory; 
import com.opengamma.analytics.math.interpolation.Interpolator1DFactory; 
import com.opengamma.analytics.math.matrix.DoubleMatrix1D; 
import com.opengamma.analytics.math.matrix.DoubleMatrix2D; 
import com.opengamma.analytics.math.rootfinding.YieldCurveFittingSetup; 
import com.opengamma.analytics.math.rootfinding.YieldCurveFittingTestDataBundle; 
import com.opengamma.analytics.math.rootfinding.YieldCurveFittingTestDataBundle.TestType; 
import com.opengamma.analytics.math.rootfinding.newton.BroydenVectorRootFinder; 
import com.opengamma.analytics.math.rootfinding.newton.NewtonVectorRootFinder; 
import com.opengamma.financial.convention.frequency.SimpleFrequency; 
import com.opengamma.util.tuple.DoublesPair; 
import com.opengamma.lambdava.tuple.DoublesPair; 

/**
 * 
 */
    InstrumentSingleCurveSensitivityCalculatorTest  extends YieldCurveFittingSetup {
	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	

  static {
    if (PV_DATA.getTestType() == TestType.ANALYTIC_JACOBIAN) {
      PV_JACOBIAN_FUNCTION = new MultipleYieldCurveFinderJacobian(PV_DATA, PV_DATA.getMarketValueSensitivityCalculator());
    } else if (PV_DATA.getTestType() == TestType.FD_JACOBIAN) {
      final VectorFieldFirstOrderDifferentiator fdJacCalculator = new VectorFieldFirstOrderDifferentiator();
      PV_JACOBIAN_FUNCTION = fdJacCalculator.differentiate(PV_FUNCTION);
    } else {
      throw new IllegalArgumentException("unknown TestType " + PV_DATA.getTestType());
    }
    if (PAR_RATE_DATA.getTestType() == TestType.ANALYTIC_JACOBIAN) {
      PAR_RATE_JACOBIAN_FUNCTION = new MultipleYieldCurveFinderJacobian(PAR_RATE_DATA, PAR_RATE_DATA.getMarketValueSensitivityCalculator());
    } else if (PAR_RATE_DATA.getTestType() == TestType.FD_JACOBIAN) {
      final VectorFieldFirstOrderDifferentiator fdJacCalculator = new VectorFieldFirstOrderDifferentiator();
      PAR_RATE_JACOBIAN_FUNCTION = fdJacCalculator.differentiate(PAR_RATE_FUNCTION);
    } else {
      throw new IllegalArgumentException("unknown TestType " + PAR_RATE_DATA.getTestType());
    }
    PV_YIELD_CURVE_NODES = ROOT_FINDER.getRoot(PV_FUNCTION, PV_JACOBIAN_FUNCTION, PV_DATA.getStartPosition());
    PV_JACOBIAN = PV_JACOBIAN_FUNCTION.evaluate(PV_YIELD_CURVE_NODES);
    PAR_RATE_YIELD_CURVE_NODES = ROOT_FINDER.getRoot(PAR_RATE_FUNCTION, PAR_RATE_JACOBIAN_FUNCTION, PAR_RATE_DATA.getStartPosition());
    PAR_RATE_JACOBIAN = PAR_RATE_JACOBIAN_FUNCTION.evaluate(PAR_RATE_YIELD_CURVE_NODES);
    PAR_RATE_CURVES = getYieldCurveMap(PAR_RATE_DATA, PAR_RATE_YIELD_CURVE_NODES);
    PAR_RATE_ALL_CURVES = getAllCurves(PAR_RATE_DATA, PAR_RATE_CURVES);
    PV_CURVES = getYieldCurveMap(PV_DATA, PV_YIELD_CURVE_NODES);
    PV_ALL_CURVES = getAllCurves(PV_DATA, PV_CURVES);
    final double[] couponSensitivityArray = new double[PV_DATA.getNumInstruments()];
    for (int i = 0; i < PV_DATA.getNumInstruments(); i++) {
      couponSensitivityArray[i] = PV_DATA.getDerivative(i).accept(PVCS, PV_ALL_CURVES);
    }
    PV_COUPON_SENSITIVITY = new DoubleMatrix1D(couponSensitivityArray);
  }

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  static {
    if (PV_DATA.getTestType() == TestType.ANALYTIC_JACOBIAN) {
      PV_JACOBIAN_FUNCTION = new MultipleYieldCurveFinderJacobian(PV_DATA, PV_DATA.getMarketValueSensitivityCalculator());
    } else if (PV_DATA.getTestType() == TestType.FD_JACOBIAN) {
      final VectorFieldFirstOrderDifferentiator fdJacCalculator = new VectorFieldFirstOrderDifferentiator();
      PV_JACOBIAN_FUNCTION = fdJacCalculator.differentiate(PV_FUNCTION);
    } else {
      throw new IllegalArgumentException("unknown TestType " + PV_DATA.getTestType());
    }
    if (PAR_RATE_DATA.getTestType() == TestType.ANALYTIC_JACOBIAN) {
      PAR_RATE_JACOBIAN_FUNCTION = new MultipleYieldCurveFinderJacobian(PAR_RATE_DATA, PAR_RATE_DATA.getMarketValueSensitivityCalculator());
    } else if (PAR_RATE_DATA.getTestType() == TestType.FD_JACOBIAN) {
      final VectorFieldFirstOrderDifferentiator fdJacCalculator = new VectorFieldFirstOrderDifferentiator();
      PAR_RATE_JACOBIAN_FUNCTION = fdJacCalculator.differentiate(PAR_RATE_FUNCTION);
    } else {
      throw new IllegalArgumentException("unknown TestType " + PAR_RATE_DATA.getTestType());
    }
    PV_YIELD_CURVE_NODES = ROOT_FINDER.getRoot(PV_FUNCTION, PV_JACOBIAN_FUNCTION, PV_DATA.getStartPosition());
    PV_JACOBIAN = PV_JACOBIAN_FUNCTION.evaluate(PV_YIELD_CURVE_NODES);
    PAR_RATE_YIELD_CURVE_NODES = ROOT_FINDER.getRoot(PAR_RATE_FUNCTION, PAR_RATE_JACOBIAN_FUNCTION, PAR_RATE_DATA.getStartPosition());
    PAR_RATE_JACOBIAN = PAR_RATE_JACOBIAN_FUNCTION.evaluate(PAR_RATE_YIELD_CURVE_NODES);
    PAR_RATE_CURVES = getYieldCurveMap(PAR_RATE_DATA, PAR_RATE_YIELD_CURVE_NODES);
    PAR_RATE_ALL_CURVES = getAllCurves(PAR_RATE_DATA, PAR_RATE_CURVES);
    PV_CURVES = getYieldCurveMap(PV_DATA, PV_YIELD_CURVE_NODES);
    PV_ALL_CURVES = getAllCurves(PV_DATA, PV_CURVES);
    final double[] couponSensitivityArray = new double[PV_DATA.getNumInstruments()];
    for (int i = 0; i < PV_DATA.getNumInstruments(); i++) {
      couponSensitivityArray[i] = PV_DATA.getDerivative(i).accept(PVCS, PV_ALL_CURVES);
    }
    PV_COUPON_SENSITIVITY = new DoubleMatrix1D(couponSensitivityArray);
  }


}
