/**
 * Copyright (C) 2009 - present by OpenGamma Inc. and the OpenGamma group of companies
 * 
 * Please see distribution for license.
 */
package com.opengamma.analytics.math.rootfinding; 

import static org.testng.AssertJUnit.assertEquals; 
import static org.threeten.bp.temporal.ChronoUnit.MONTHS; 

import java.util.Arrays; 
import java.util.HashMap; 
import java.util.LinkedHashMap; 
import java.util.List; 
import java.util.Map; 

import org.apache.commons.lang.Validate; 
import org.slf4j.Logger; 
import org.threeten.bp.Period; 

import cern.jet.random.engine.MersenneTwister; 
import cern.jet.random.engine.MersenneTwister64; 
import cern.jet.random.engine.RandomEngine; 

import com.opengamma.analytics.financial.forex.method.FXMatrix; 
import com.opengamma.analytics.financial.instrument.index.IborIndex; 
import com.opengamma.analytics.financial.instrument.index.IndexON; 
import com.opengamma.analytics.financial.interestrate.InstrumentDerivative; 
import com.opengamma.analytics.financial.interestrate.InstrumentDerivativeVisitor; 
import com.opengamma.analytics.financial.interestrate.MultipleYieldCurveFinderDataBundle; 
import com.opengamma.analytics.financial.interestrate.MultipleYieldCurveFinderFunction; 
import com.opengamma.analytics.financial.interestrate.MultipleYieldCurveFinderJacobian; 
import com.opengamma.analytics.financial.interestrate.RateReplacingInterestRateDerivativeVisitor; 
import com.opengamma.analytics.financial.interestrate.YieldCurveBundle; 
import com.opengamma.analytics.financial.interestrate.annuity.derivative.Annuity; 
import com.opengamma.analytics.financial.interestrate.annuity.derivative.AnnuityCouponFixed; 
import com.opengamma.analytics.financial.interestrate.annuity.derivative.AnnuityCouponIbor; 
import com.opengamma.analytics.financial.interestrate.annuity.derivative.AnnuityCouponIborSpread; 
import com.opengamma.analytics.financial.interestrate.cash.derivative.Cash; 
import com.opengamma.analytics.financial.interestrate.fra.ForwardRateAgreement; 
import com.opengamma.analytics.financial.interestrate.future.derivative.InterestRateFuture; 
import com.opengamma.analytics.financial.interestrate.payments.ForexForward; 
import com.opengamma.analytics.financial.interestrate.payments.derivative.CouponFixed; 
import com.opengamma.analytics.financial.interestrate.payments.derivative.CouponIborSpread; 
import com.opengamma.analytics.financial.interestrate.payments.derivative.CouponOIS; 
import com.opengamma.analytics.financial.interestrate.payments.derivative.PaymentFixed; 
import com.opengamma.analytics.financial.interestrate.swap.derivative.CrossCurrencySwap; 
import com.opengamma.analytics.financial.interestrate.swap.derivative.FixedFloatSwap; 
import com.opengamma.analytics.financial.interestrate.swap.derivative.FloatingRateNote; 
import com.opengamma.analytics.financial.interestrate.swap.derivative.SwapFixedCoupon; 
import com.opengamma.analytics.financial.interestrate.swap.derivative.TenorSwap; 
import com.opengamma.analytics.financial.model.interestrate.curve.YieldAndDiscountCurve; 
import com.opengamma.analytics.financial.model.interestrate.curve.YieldCurve; 
import com.opengamma.analytics.math.curve.InterpolatedDoublesCurve; 
import com.opengamma.analytics.math.differentiation.FiniteDifferenceType; 
import com.opengamma.analytics.math.differentiation.VectorFieldFirstOrderDifferentiator; 
import com.opengamma.analytics.math.function.Function1D; 
import com.opengamma.analytics.math.interpolation.Interpolator1D; 
import com.opengamma.analytics.math.matrix.DoubleMatrix1D; 
import com.opengamma.analytics.math.matrix.DoubleMatrix2D; 
import com.opengamma.analytics.math.rootfinding.YieldCurveFittingTestDataBundle.TestType; 
import com.opengamma.analytics.math.rootfinding.newton.NewtonVectorRootFinder; 
import com.opengamma.financial.convention.businessday.BusinessDayConventionFactory; 
import com.opengamma.financial.convention.calendar.MondayToFridayCalendar; 
import com.opengamma.financial.convention.daycount.DayCountFactory; 
import com.opengamma.financial.convention.frequency.SimpleFrequency; 
import com.opengamma.util.money.Currency; 
import com.opengamma.util.money.CurrencyAmount; 
import com.opengamma.util.monitor.OperationTimer; 
import com.opengamma.util.tuple.DoublesPair; 
import com.opengamma.lambdava.tuple.DoublesPair; 

/**
 * 
 */
    YieldCurveFittingSetup {
	
  /** Random number generator */
  

	
  /** Replaces rates */
  

	
  

	
  /** FX rates */
  

	
  

	
  

	

  /** Accuracy */
  

	
  /** Number of steps */
  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  /**
   * makes a very simple FRA with  payment time, fixing time and fixing period start being identical and an amount tau before fixing period end. The payment and fixing year fractions are
   * Identically equal to tau.
   * @param time The fixing period end (the last relevant date for the FRA)
   * @param paymentFreq for a 3M FRA the payment freq is quarterly
   * @param fundCurveName Name of funding curve
   * @param indexCurveName Name of index curve
   * @param rate The FRA rate
   * @param notional the notional amount
   * @return A FRA
   */
  

	

  

	

  

	

  

	

  

	

  

	

  

	

  /**
   * Sets up a simple Floating rate note to test the analytics
   * @param notional An amount in a currency
   * @param nYears time to maturity in years
   * @param freq Frequency of payments
   * @param discountCurve Name of discount curve
   * @param indexCurve Name of index curve
   * @param spread the spread paid
   * @return A FRN
   */
  

	

  

	

  

	

  

	

  


}
