/**
 * Copyright (C) 2009 - present by OpenGamma Inc. and the OpenGamma group of companies
 *
 * Please see distribution for license.
 */
package com.opengamma.engine.view; 

import java.util.HashSet; 
import java.util.Set; 
import java.util.concurrent.Semaphore; 
import java.util.concurrent.atomic.AtomicLong; 
import java.util.concurrent.atomic.AtomicReference; 
import java.util.concurrent.locks.ReentrantLock; 

import org.slf4j.Logger; 
import org.slf4j.LoggerFactory; 
import org.springframework.context.Lifecycle; 
import org.threeten.bp.Instant; 

import com.opengamma.OpenGammaRuntimeException; 
import com.opengamma.engine.marketdata.MarketDataInjector; 
import com.opengamma.engine.marketdata.MarketDataPermissionProvider; 
import com.opengamma.engine.value.ValueRequirement; 
import com.opengamma.engine.view.calc.EngineResourceManagerInternal; 
import com.opengamma.engine.view.calc.SingleComputationCycle; 
import com.opengamma.engine.view.calc.ViewComputationJob; 
import com.opengamma.engine.view.calc.ViewCycle; 
import com.opengamma.engine.view.calc.ViewCycleMetadata; 
import com.opengamma.engine.view.client.ViewDeltaResultCalculator; 
import com.opengamma.engine.view.compilation.CompiledViewDefinitionWithGraphsImpl; 
import com.opengamma.engine.view.execution.ViewCycleExecutionOptions; 
import com.opengamma.engine.view.execution.ViewExecutionOptions; 
import com.opengamma.engine.view.listener.ViewResultListener; 
import com.opengamma.engine.view.permission.ViewPermissionProvider; 
import com.opengamma.id.ObjectId; 
import com.opengamma.id.UniqueId; 
import com.opengamma.livedata.UserPrincipal; 
import com.opengamma.util.ArgumentChecker; 
import com.opengamma.util.tuple.Pair; 
import com.opengamma.lambdava.tuple.Pair; 

/**
 * Default implementation of {@link ViewProcess}.
 */
    ViewProcessImpl   {
	

  

	

  

	
  

	
  

	
  

	
  

	
  

	

  

	

  /**
   * Manages access to critical regions of the process. Note that the use of {@link Semaphore} rather than, for example, {@link ReentrantLock} allows one thread to acquire the lock and another thread
   * to release it as part of the same sequence of operations. This could be important for {@link #suspend()} and {@link #resume()}.
   */
  

	

  

	

  

	

  

	
  

	

  

	

  

	
  

	

  /**
   * Constructs an instance.
   *
   * @param viewProcessId the unique identifier of the view process, not null
   * @param viewDefinitionId the name of the view definition, not null
   * @param executionOptions the view execution options, not null
   * @param viewProcessContext the process context, not null
   * @param cycleManager the view cycle manager, not null
   * @param cycleObjectId the object identifier of cycles, not null
   */
  

	

  //-------------------------------------------------------------------------
  

	

  

	

  

	

  

	

  

	

  

	

  

	

  //-------------------------------------------------------------------------
  // Lifecycle
  //-------------------------------------------------------------------------

  

	

  

	

  

	

  

	

  

	

  //-------------------------------------------------------------------------
  

	

  //-------------------------------------------------------------------------
  /**
   * Gets the execution log mode source.
   *
   * @return the execution log mode source, not null
   */
  

	

  //-------------------------------------------------------------------------
  

	

  

	

  

	

  

	

  

	

  

	

  

	

  

	

  //-------------------------------------------------------------------------
  

	

  

	

  /**
   * Sets the current view process state.
   *
   * @param state the new view process state
   */
  

	

  /**
   * Sets the current view computation job.
   * <p>
   * External visibility for testing.
   *
   * @return the current view computation job
   */
  

	

  /**
   * Sets the current computation job
   *
   * @param computationJob the current computation job
   */
  

	

  /**
   * Gets the current computation job's thread
   * <p>
   * External visibility for testing.
   *
   * @return the current computation job thread
   */
  

	

  /**
   * Sets the current computation job's thread
   *
   * @param computationJobThread the current computation job thread
   */
  

	

  

	

  

	

  /**
   * Attaches a listener to the view process.
   * <p>
   * The method operates with set semantics, so duplicate notifications for the same listener have no effect.
   *
   * @param listener the listener, not null
   * @return the permission provider for the process, not null
   */
  

	

  /**
   * Removes a listener from the view process. Removal of the last listener generating execution demand will cause the process to stop.
   * <p>
   * The method operates with set semantics, so duplicate notifications for the same listener have no effect.
   *
   * @param listener the listener, not null
   */
  

	

  

	

  

	

  

	

  /**
   * Starts the background job responsible for running computation cycles for this view process.
   */
  

	

  /**
   * Instructs the background computation job to finish. The background job might actually terminate asynchronously, but any outstanding result will be discarded. A replacement background computation
   * job may be started immediately.
   */
  

	

  

	

  


}
