/**
 * Copyright (C) 2009 - present by OpenGamma Inc. and the OpenGamma group of companies
 * 
 * Please see distribution for license.
 */
package com.opengamma.engine.view; 

import java.util.ArrayList; 
import java.util.Collection; 
import java.util.Collections; 
import java.util.HashMap; 
import java.util.List; 
import java.util.Map; 
import java.util.Timer; 
import java.util.concurrent.ConcurrentHashMap; 
import java.util.concurrent.ConcurrentMap; 
import java.util.concurrent.ExecutorService; 
import java.util.concurrent.Future; 
import java.util.concurrent.TimeUnit; 
import java.util.concurrent.TimeoutException; 
import java.util.concurrent.atomic.AtomicLong; 
import java.util.concurrent.locks.ReentrantLock; 

import org.slf4j.Logger; 
import org.slf4j.LoggerFactory; 

import com.opengamma.DataNotFoundException; 
import com.opengamma.OpenGammaRuntimeException; 
import com.opengamma.core.config.ConfigSource; 
import com.opengamma.engine.depgraph.DependencyGraphBuilderFactory; 
import com.opengamma.engine.function.CompiledFunctionService; 
import com.opengamma.engine.function.resolver.FunctionResolver; 
import com.opengamma.engine.marketdata.MarketDataInjector; 
import com.opengamma.engine.marketdata.NamedMarketDataSpecificationRepository; 
import com.opengamma.engine.marketdata.OverrideOperationCompiler; 
import com.opengamma.engine.marketdata.resolver.MarketDataProviderResolver; 
import com.opengamma.engine.view.cache.ViewComputationCacheSource; 
import com.opengamma.engine.view.calc.DependencyGraphExecutorFactory; 
import com.opengamma.engine.view.calc.EngineResourceManagerImpl; 
import com.opengamma.engine.view.calc.EngineResourceManagerInternal; 
import com.opengamma.engine.view.calc.SingleComputationCycle; 
import com.opengamma.engine.view.calc.ViewResultListenerFactory; 
import com.opengamma.engine.view.calc.stats.GraphExecutorStatisticsGathererProvider; 
import com.opengamma.engine.view.calcnode.JobDispatcher; 
import com.opengamma.engine.view.client.ViewClient; 
import com.opengamma.engine.view.client.ViewClientImpl; 
import com.opengamma.engine.view.event.ViewProcessorEventListenerRegistry; 
import com.opengamma.engine.view.execution.ViewExecutionFlags; 
import com.opengamma.engine.view.execution.ViewExecutionOptions; 
import com.opengamma.engine.view.listener.ViewResultListener; 
import com.opengamma.engine.view.permission.ViewPermissionProvider; 
import com.opengamma.id.ObjectId; 
import com.opengamma.id.UniqueId; 
import com.opengamma.livedata.UserPrincipal; 
import com.opengamma.util.ArgumentChecker; 
import com.opengamma.util.monitor.OperationTimer; 
import com.opengamma.util.tuple.Pair; 
import com.opengamma.lambdava.tuple.Pair; 

/**
 * Default implementation of {@link ViewProcessor}.
 */
    ViewProcessorImpl   {
	

  

	

  

	
  

	
  

	

  

	
  

	
  

	
  

	

  

	

  

	

  // Injected inputs
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	
  

	

  // State
  /**
   * ConcurrentHashMap to allow access for querying processes independently and concurrently to client attachment.
   */
  

	
  

	

  

	

  

	

  /**
   * The view processor event listener registry
   */
  

	

  

	
  

	
  
  // TODO: would it be easier to pass in a ViewProcessContext to this constructor ?

  

	

  //-------------------------------------------------------------------------
  

	

  

	

  

	
  
  //-------------------------------------------------------------------------
  

	

  

	

  

	

  /**
   * Obtains a shared view process matching the given arguments, creating the process if necessary, and associates the
   * client with that process.
   *
   * @param clientId  the unique identifier of the client, not null
   * @param listener  the process listener, not null
   * @param viewDefinitionId  the id of the view definition, not null
   * @param executionOptions  the view execution options, not null
   * @return the permission provider to be used for access control, not null
   */
  

	

  /**
   * Obtains a new, private view process, and associates the client with that process.
   *
   * @param clientId  the unique identifier of the client, not null
   * @param listener  the process listener, not null
   * @param viewDefinitionId  the id of the view definition, not null
   * @param executionOptions  the view execution options, not null
   * @return the permission provider to be used for access control, not null
   */
  

	

  /**
   * Obtains an existing view process, and associates the client with that process.
   *
   * @param clientId  the unique identifier of the client, not null
   * @param listener  the process listener, not null
   * @param processId  the unique identifier of the existing process, not null
   * @return the permission provider to be used for access control, not null
   */
  

	

  

	

  /**
   * Removes the association, if any, between a client and a view process. This may allow the view process to be
   * terminated and removed.
   *
   * @param clientId  the unique identifier of the client, not null
   */
  

	

  

	

  

	
  
  

	
  
  

	
  
  //-------------------------------------------------------------------------
  
  /**
   * Gets the live data override injector for the view process currently associated with a client.
   * 
   * @param clientId  the unique identifier of the client, not null
   * @return the live data override injector, not null
   * @throws IllegalStateException if the client is not associated with a view process
   */
  

	
  
  /**
   * Gets the view definition being operated on by the process associated with a client.
   * 
   * @param clientId  the unique identifier of the client, not null
   * @return the view definition, not null
   * @throws IllegalStateException if the client is not associated with a view process
   */
  

	
  
  /**
   * Requests that a computation cycle be run, even if none of the other triggers have fired since the last cycle.
   * 
   * @param clientId  the unique identifier of the client, not null
   * @throws IllegalStateException if the client is not associated with a view process
   */
  

	
  
  

	
  
  //-------------------------------------------------------------------------
  

	

  

	
  
  /**
   * Must be called by the client when it is terminated to ensure that associated resources are cleaned up.
   * 
   * @param clientId  the unique identifier, not null
   */
  

	
  
  //-------------------------------------------------------------------------
  

	
  
  

	
  
  //-------------------------------------------------------------------------
  

	
  
  //-------------------------------------------------------------------------
  /**
   * Intended for testing.
   * 
   * @param viewClientId  the unique identifier of the view client, not null
   * @return the view process, not null
   */
  

	
  
  

	
  
  

	

  

	

  //-------------------------------------------------------------------------
  // Lifecycle
  //-------------------------------------------------------------------------
  
  

	

  

	
  
  

	

  

	
  
  //-------------------------------------------------------------------------
      ViewProcessDescription {
		
    
    

		
    

		
    
    

		

    

		

    


	}


}
