/**
 * Copyright (C) 2009 - present by OpenGamma Inc. and the OpenGamma group of companies
 *
 * Please see distribution for license.
 */
package com.opengamma.engine.view.calcnode; 

import java.util.Collection; 
import java.util.HashSet; 
import java.util.Iterator; 
import java.util.Map; 
import java.util.Queue; 
import java.util.Set; 
import java.util.concurrent.CancellationException; 
import java.util.concurrent.ConcurrentHashMap; 
import java.util.concurrent.ConcurrentLinkedQueue; 
import java.util.concurrent.ConcurrentMap; 
import java.util.concurrent.ConcurrentSkipListMap; 
import java.util.concurrent.ExecutorService; 
import java.util.concurrent.Executors; 
import java.util.concurrent.atomic.AtomicInteger; 

import org.slf4j.Logger; 
import org.slf4j.LoggerFactory; 

import com.opengamma.util.ArgumentChecker; 
import com.opengamma.util.async.AsynchronousExecution; 
import com.opengamma.util.async.AsynchronousResult; 
import com.opengamma.util.async.ResultListener; 
import com.opengamma.util.tuple.Pair; 
import com.opengamma.lambdava.tuple.Pair; 

/**
 * Base class for objects that manage a set of AbstractCalculationNodes with the intention of invoking job executions on them.
 */
    SimpleCalculationNodeInvocationContainer {
	

  /**
   * After how many failed jobs should a cleanup be attempted.
   */
  

	 // clean up after every 100 failed jobs

  /**
   * Retention period for failed jobs needs to be long enough for any tail jobs to have arrived.
   */
  

	 // 5m

  

	

  /**
   *
   */
      ExecutionReceiver {
		

    

		

    


	}

	

      JobEntry {
		

    

		
    

		
    

		
    

		

    

		

    

		

    

		

    

		

    /**
     * This is only called from a single thread - doing the addJob operation - once it has been called once, another thread will manipulate the block count (e.g. if a job is finishing) and possibly
     * spawn the job. Note that we initialize the count to two so that the job does not get spawned prematurely until the addJob thread has processed the required job list and performs a decrement to
     * clear the additional value.
     */
    

		

    /**
     * Decrements the block count.
     *
     * @return true when the count reaches zero
     */
    

		

    

		

    

		

    


	}

	

      PartialJobEntry {
		

    

		
    

		
    

		

    

		

    

		

    

		

    


	}

	

      JobExecution {
		

     enum  Status {
       ,   ,  ;}

		

    

		
    

		
    

		
    

		
    

		

    

		

    

		

    

		

    

		

    

		

    

		

    

		

    // Caller must own the monitor
    

		

    // Caller must own the monitor
    


	}

	

  /**
   * The nodes that are available for execution.
   */
  

	

  /**
   * The total number of nodes that are in the container.
   */
  

	

  /**
   * The set of jobs that are either running, in the runnable queue or blocked by other jobs.
   */
  

	

  /**
   * The set of failed jobs. Anything not in this set or {@link #_executions} has completed successfully.
   */
  

	
  

	

  /**
   * The queue of runnable jobs. Each are jobs that have been received and are ready to run but have not been started, probably because they are waiting for a node to become available. When nodes are
   * available they will take partial jobs from the {@link _partialJobs} queue in preference to this queue.
   */
  

	
  /**
   * The queue of partially executed jobs. Each represents a piece of work that was started, became blocked on something external, but is now ready to run again. Note that the resume may not take
   * place on the original node and/or original thread. This is to avoid a potential bottleneck. Nothing should be retaining thread/node specific state (other than the saved node state) so this should
   * not be a problem.
   */
  

	

  

	

  

	

  

	

  

	

  

	

  

	

  /**
   * Removes a node if one is available.
   *
   * @return the node removed from the live set, or null if there are none to remove
   */
  

	

  /**
   * Returns the total number of nodes in this invocation set. This includes those in the available set and those that are currently busy executing jobs.
   *
   * @return the total number of nodes
   */
  

	

  /**
   * Returns the number of available nodes in the set. Note that the structure that holds the nodes may have quite a costly size operation.
   *
   * @return the number of available nodes in the set
   */
  

	

  /**
   * Returns the total number of jobs enqueued at this node. This includes both runnable, partial and blocked jobs. Note that the structure that holds the jobs may have quite a costly size operation.
   *
   * @return the number of jobs
   */
  

	

  /**
   * Returns the number of jobs enqueued at this node that are available to start. This includes both runnable, partial and blocked jobs. Note that the structure that holds the jobs may have quite a
   * costly size operation.
   *
   * @return the number of jobs
   */
  

	

  /**
   * Returns the number of jobs enqueued at this node that have been partially completed and are ready for continuation. Note that the structure that holds the jobs may have quite a costly size
   * operation.
   *
   * @return the number of jobs
   */
  

	

  

	

  

	

  

	

  

	

  

	

  

	

  /**
   * Adds a job to the runnable queue, spawning a worker thread if a node is supplied or one is available.
   */
  

	

  

	

  

	

  

	

  /**
   * Adds jobs to the runnable queue, spawning a worker thread if a node is supplied or one is available. Jobs must be added in dependency order - i.e. a job must be submitted before any that require
   * it. This is to simplify retention of job status as we only need to track jobs that are still running or have failed which saves a lot of housekeeping overhead.
   *
   * @param job job to run, not null
   * @param receiver execution status receiver, not null
   * @param node optional node to start a worker thread with
   */
  

	

  

	

  /**
   * Executes jobs from the runnable and partially-run queues until both are empty.
   *
   * @param node Node to run on, not null
   * @param job The first job to run, not null if resumeJob is null
   * @param resumeJob the first job to run, not null if newJob is null
   */
  

	

  

	

  


}
