/**
 * Copyright (C) 2009 - present by OpenGamma Inc. and the OpenGamma group of companies
 * 
 * Please see distribution for license.
 */
package com.opengamma.engine.view.calcnode.stats; 

import java.util.ArrayList; 
import java.util.Collections; 
import java.util.Comparator; 
import java.util.HashMap; 
import java.util.List; 
import java.util.Map; 
import java.util.Queue; 
import java.util.Set; 
import java.util.concurrent.ConcurrentHashMap; 
import java.util.concurrent.ConcurrentLinkedQueue; 
import java.util.concurrent.ConcurrentMap; 

import org.fudgemsg.FudgeContext; 
import org.fudgemsg.FudgeMsg; 
import org.fudgemsg.FudgeMsgFactory; 
import org.fudgemsg.MutableFudgeMsg; 
import org.slf4j.Logger; 
import org.slf4j.LoggerFactory; 

import com.opengamma.util.ArgumentChecker; 
import com.opengamma.util.tuple.Pair; 
import com.opengamma.lambdava.tuple.Pair; 

/**
 * Central point for function statistics.
 * <p>
 * This acts as a local gatherer, central point for remote nodes and supplier of data.
 */
    FunctionCosts   {
	

  /** Logger. */
  

	
  /**
   * Name used for the mean.
   */
  

	

  /**
   * The statistics per configuration.
   */
  

	
  /**
   * The items that persistence is dealing with.
   */
  

	
  /**
   * The persistent storage.
   */
  

	
  /**
   * The mean statistics as persisted.
   */
  

	

  /**
   * Constructor using an in-memory master.
   */
  

	

  /**
   * Constructor.
   * 
   * @param costsMaster  the costs master, not null
   */
  

	

  /**
   * Loads the mean statistics.
   */
  

	

  //-------------------------------------------------------------------------
  /**
   * Gathers statistics from the central node and records them.
   * 
   * @param configurationName  the configuration name, not null
   * @param functionId  the function id, not null
   * @param invocationCount  the number of invocations the data is for
   * @param executionNanos  the execution time, in nanoseconds, of the invocation(s)
   * @param dataInputBytes  the mean data input, bytes per input node, or {@code NaN} to mean statistics aren't available
   * @param dataOutputBytes  the mean data output, bytes per output node, or {@code NaN} to mean statistics aren't available
   */
  

	

  //-------------------------------------------------------------------------
  /**
   * Gets statistics for a configuration.
   * 
   * @param configurationName  the configuration name, not null
   * @return the statistics, not null
   */
  

	

  /**
   * Gets statistics for a function.
   * 
   * @param configurationName  the configuration name, not null
   * @param functionId  the function id, not null
   * @return the statistics, not null
   */
  

	

  /**
   * Returns the defined set of configurations.
   * 
   * @return the configurations, not null
   */
  

	

  /**
   * Loads the statistics.
   * 
   * @param configurationName the configuration name, not null
   * @param functionId the function id, not null
   * @return the statistics, not null
   */
  /* package */ 

	

  //-------------------------------------------------------------------------
  /**
   * Creates a runnable to write the statistics.
   * 
   * @return the runnable, not null
   */
  <<<<<<< E:\ssmerge\fstmerge_tmp1502139584330\fstmerge_var1_4415867017998081705.java
public Runnable createPersistenceWriter() {
    // the statistics are added to a concurrent collection, which is processed here

    return new Runnable() {
      @Override
      public void run() {
        s_logger.info("Persisting function execution statistics");
        final FunctionInvocationStatistics meanStatistics = _meanStatistics;
        final long lastUpdate = meanStatistics.getLastUpdateNanos();
        // [PLAT-882] Temporary hack until JMX support is property implemented
        final Map<String, FudgeMsg> report = new HashMap<String, FudgeMsg>();
        // store updates and calculate mean
        int count = 1;
        double invocationCost = meanStatistics.getInvocationCost();
        double dataInputCost = meanStatistics.getDataInputCost();
        double dataOutputCost = meanStatistics.getDataOutputCost();
        for (final Pair<String, FunctionInvocationStatistics> pair : _persistedItems) {
          final FunctionInvocationStatistics stats = pair.getSecond();
          if (stats.getLastUpdateNanos() > lastUpdate) {
            // store
            s_logger.debug("Storing {}/{}", pair.getFirst(), stats.getFunctionId());
            final FunctionCostsDocument doc = new FunctionCostsDocument(pair.getFirst(), stats.getFunctionId());
            stats.populateDocument(doc);
            _costsMaster.store(doc);
            // calculate mean
            invocationCost += stats.getInvocationCost();
            dataInputCost += stats.getDataInputCost();
            dataOutputCost += stats.getDataOutputCost();
            count++;
            // [PLAT-882] Temporary hack until JMX support is properly implemented
            if (s_logger.isInfoEnabled()) {
              report.put(stats.getFunctionId() + "\t" + pair._1(), stats.toFudgeMsg(FudgeContext.GLOBAL_DEFAULT));
            }
          }
        }
        meanStatistics.setCosts(invocationCost / count, dataInputCost / count, dataOutputCost / count);
        if (count > 1) {
          s_logger.debug("Storing new mean statistics {}", meanStatistics);
          final FunctionCostsDocument doc = new FunctionCostsDocument(MEAN_STATISTICS, MEAN_STATISTICS);
          meanStatistics.populateDocument(doc);
          _costsMaster.store(doc);
        }
        // [PLAT-882] Temporary hack until JMX support is property implemented
        if (s_logger.isInfoEnabled()) {
          final List<String> keys = new ArrayList<String>(report.keySet());
          Collections.sort(keys, new Comparator<String>() {
            @Override
            public int compare(final String a, final String b) {
              final double c = report.get(a).getDouble("invocationCost") - report.get(b).getDouble("invocationCost");
              if (c < 0) {
                return -1;
              } else if (c > 0) {
                return 1;
              } else {
                return 0;
              }
            }
          });
          for (final String key : keys) {
            s_logger.info("{}\t{}", key, report.get(key));
          }
        }
      }
    };
  }
=======
>>>>>>> E:\ssmerge\fstmerge_tmp1502139584330\fstmerge_var2_1784504006547601385.java


	

  //-------------------------------------------------------------------------
  

	

  //-------------------------------------------------------------------------
  // For debug purposes only, remove when PLAT-882 is complete
  


}
