/**
 * Copyright (C) 2009 - present by OpenGamma Inc. and the OpenGamma group of companies
 *
 * Please see distribution for license.
 */
package com.opengamma.util.db; 

import static com.opengamma.util.db.HibernateDbUtils.fixSQLExceptionCause; 

import java.sql.Timestamp; 

import javax.sql.DataSource; 

import org.hibernate.SessionFactory; 
import org.slf4j.Logger; 
import org.slf4j.LoggerFactory; 
import org.springframework.dao.DataAccessException; 
import org.springframework.dao.DataIntegrityViolationException; 
import org.springframework.jdbc.core.simple.SimpleJdbcTemplate; 
import org.springframework.orm.hibernate3.HibernateCallback; 
import org.springframework.orm.hibernate3.HibernateTemplate; 
import org.springframework.transaction.PlatformTransactionManager; 
import org.springframework.transaction.TransactionException; 
import org.springframework.transaction.TransactionStatus; 
import org.springframework.transaction.support.TransactionCallback; 
import org.springframework.transaction.support.TransactionTemplate; 
import org.threeten.bp.Clock; 
import org.threeten.bp.Instant; 
import org.threeten.bp.ZoneId; 

import com.opengamma.util.ArgumentChecker; 
import com.opengamma.util.Connector; 
import com.opengamma.util.OpenGammaClock; 
import com.opengamma.util.ReflectionUtils; 
import com.opengamma.util.time.DateUtils; 

/**
 * Connector used to access SQL databases.
 * <p>
 * This class provides a simple-to-setup and simple-to-use way to access databases. It can be configured for access via JDBC, Hibernate or both. The main benefit is simpler configuration, especially
 * if that configuration is in XML.
 * <p>
 * This class is usually configured using the associated factory bean.
 */
    DbConnector   {
	

  

	
  private static final Logger s_logger = LoggerFactory.getLogger(DbConnector.class);

	

  static {
    DateUtils.initTimeZone();
  }

	

  /**
   * The configuration name.
   */
  

	
  /**
   * The data source.
   */
  

	
  /**
   * The dialect.
   */
  

	
  /**
   * The JDBC template.
   */
  

	
  /**
   * The Hibernate template.
   */
  

	
  /**
   * The transaction template.
   */
  

	

  /**
   * The current {@link #now} time. This is a cached value based on the {@link System#nanoTime} clock for up to {@link #NOW_CACHE_TTL} to avoid hammering the database with "SELECT NOW" queries.
   */
  

	

  /**
   * The {@link System#nanoTime} clock when {@link #_now} was retrieved.
   */
  

	

  /**
   * Creates an instance.
   * 
   * @param name the configuration name, not null
   * @param dialect the database dialect, not null
   * @param dataSource the data source, not null
   * @param jdbcTemplate the JDBC template, not null
   * @param hibernateTemplate the Hibernate template, may be null
   * @param transactionTemplate the transaction template, not null
   */
  

	

  //-------------------------------------------------------------------------
  

	

  

	

  //-------------------------------------------------------------------------
  /**
   * Gets the data source.
   * 
   * @return the data source, not null
   */
  

	

  /**
   * Gets the database dialect.
   * 
   * @return the database dialect, not null
   */
  

	

  /**
   * Gets the JDBC template.
   * 
   * @return the JDBC template, not null
   */
  

	

  //-------------------------------------------------------------------------
  /**
   * Gets the Hibernate session factory.
   * <p>
   * This is shared between all users of this object and must not be further configured.
   * 
   * @return the Hibernate session factory, may be null
   */
  

	

  /**
   * Gets the shared Hibernate template.
   * <p>
   * This is shared between all users of this object and must not be further configured.
   * 
   * @return the Hibernate template, null if the session factory is null
   */
  

	

  /**
   * Gets the simple Hibernate transaction template.
   * 
   * @return the Hibernate transaction template, not null
   */
  

	

  /**
   * Gets the retrying Hibernate transaction template.
   * 
   * @param retries how many maximum retires should be tried
   * @return the retrying Hibernate transaction template, not null
   */
  

	

  //-------------------------------------------------------------------------
  /**
   * Gets the transaction manager.
   * <p>
   * This is shared between all users of this object and must not be further configured.
   * 
   * @return the transaction manager, may be null
   */
  

	

  /**
   * Gets the transaction template.
   * <p>
   * This is shared between all users of this object and must not be further configured.
   * 
   * @return the transaction template, may be null
   */
  

	

  /**
   * Gets the retrying transaction template.
   * <p>
   * 
   * @param retries how many maximum retires should be tried
   * @return the retrying transaction template
   */
  

	

  //-------------------------------------------------------------------------
  /**
   * Gets the current instant using the database clock.
   * 
   * @return the current database instant, may be null
   */
  

	

  /**
   * Returns a time-source based on the current database clock.
   * <p>
   * This can be used to obtain the current instant by calling {@link Instant#now(Clock)}.
   * 
   * @return the database time-source, may be null
   */
  

	

  //-------------------------------------------------------------------------
  

	

  //-------------------------------------------------------------------------
  /**
   * Returns a description of this object suitable for debugging.
   * 
   * @return the description, not null
   */
  

	

  //-------------------------------------------------------------------------
  /**
   * A transaction template that retries the underlying Spring-based template.
   */
      TransactionTemplateRetrying {
		
    

		
    

		

    

		

    /**
     * Executes the template, which will retry the code in the event of failure.
     * 
     * @param <T> the type of the result
     * @param action the underlying Spring-based template containing the action to perform, not null
     * @return the result of the underlying template
     * @throws TransactionException if an error occurs
     */
    <<<<<<< E:\ssmerge\fstmerge_tmp1502139605036\fstmerge_var1_7017699383898339000.java
public <T> T execute(TransactionCallback<T> action) throws TransactionException {
      // retry to handle concurrent conflicting inserts into unique content tables
      for (int retry = 0; true; retry++) {
        try {
          return _transactionTemplate.execute(action);
        } catch (DataIntegrityViolationException ex) {
          if (retry == _retries) {
            throw ex;
          }
        } catch (DataAccessException ex) {
          throw fixSQLExceptionCause(ex);
        }
      }
    }
=======
>>>>>>> E:\ssmerge\fstmerge_tmp1502139605036\fstmerge_var2_4665254544910608930.java



	}

	

  //-------------------------------------------------------------------------
  /**
   * A standard transaction template wrapping the underlying Hibernate template.
   */
      HibernateTransactionTemplate {
		
    

		
    

		

    

		

    /**
     * Executes the underlying template in a transaction.
     * 
     * @param <T> the type of the result
     * @param action the underlying Hibernate template containing the action to perform, not null
     * @return the result of the underlying template
     * @throws TransactionException if an error occurs
     */
    


	}

	

  //-------------------------------------------------------------------------
  /**
   * A transaction template that retries the underlying Hibernate template.
   */
      HibernateTransactionTemplateRetrying {
		
    

		
    

		
    

		

    

		

    /**
     * Executes the template, which will retry the code in the event of failure.
     * 
     * @param <T> the type of the result
     * @param action the underlying Hibernate template containing the action to perform, not null
     * @return the result of the underlying template
     * @throws TransactionException if an error occurs
     */
    


	}

	

  static {
    DateUtils.initTimeZone();
  }


}
