/**
 * Copyright (C) 2012 - present by OpenGamma Inc. and the OpenGamma group of companies
 * 
 * Please see distribution for license.
 */
package com.opengamma.analytics.financial.curve;

import com.opengamma.lambdava.tuple.ObjectsPair;
import com.opengamma.lambdava.tuple.Pair;

/**
 * Build of curve in several blocks with relevant Jacobian matrices.
 * Curves are using different types of interpolations (on cc rate, periodic rates and discount factors) and spread curves.
 */
public class CurveConstructionSpreadTest {

    static {
        CCY_MAP.put(CURVE_NAME_DSC_USD, CCY_USD);
        CCY_MAP.put(CURVE_NAME_FWD3_USD, CCY_USD);
        CCY_MAP.put(CURVE_NAME_FWD6_USD, CCY_USD);
    }

    static {
        DEFINITIONS_DSC_USD = getDefinitions(DSC_USD_MARKET_QUOTES, DSC_USD_GENERATORS, DSC_USD_TENOR, new Double[DSC_USD_MARKET_QUOTES.length]);
        DEFINITIONS_FWD3_USD = getDefinitions(FWD3_USD_MARKET_QUOTES, FWD3_USD_GENERATORS, FWD3_USD_TENOR, new Double[FWD3_USD_MARKET_QUOTES.length]);
        DEFINITIONS_FWD3_USD_2 = getDefinitions(FWD3_USD_MARKET_QUOTES_2, FWD3_USD_GENERATORS_2, FWD3_USD_TENOR_2, new Double[FWD3_USD_MARKET_QUOTES_2.length]);
        DEFINITIONS_FWD3_USD_3 = getDefinitions(FWD3_USD_MARKET_QUOTES_3, FWD3_USD_GENERATORS_3, FWD3_USD_TENOR_3, new Double[FWD3_USD_MARKET_QUOTES_3.length]);
        DEFINITIONS_FWD3_USD_4 = getDefinitions(FWD3_USD_MARKET_QUOTES_4, FWD3_USD_GENERATORS_4, FWD3_USD_TENOR_4, new Double[FWD3_USD_MARKET_QUOTES_4.length]);
        DEFINITIONS_FWD6_USD = getDefinitions(FWD6_USD_MARKET_QUOTES, FWD6_USD_GENERATORS, FWD6_USD_TENOR, new Double[FWD6_USD_MARKET_QUOTES.length]);
        for (int loopblock = 0; loopblock < NB_BLOCKS; loopblock++) {
            DEFINITIONS_UNITS[loopblock] = new InstrumentDefinition<?>[NB_UNITS[loopblock]][][];
            GENERATORS_UNITS[loopblock] = new GeneratorYDCurve[NB_UNITS[loopblock]][];
            NAMES_UNITS[loopblock] = new String[NB_UNITS[loopblock]][];
        }
        DEFINITIONS_UNITS[0][0] = new InstrumentDefinition<?>[][] { DEFINITIONS_DSC_USD };
        DEFINITIONS_UNITS[0][1] = new InstrumentDefinition<?>[][] { DEFINITIONS_FWD3_USD_2 };
        DEFINITIONS_UNITS[1][0] = new InstrumentDefinition<?>[][] { DEFINITIONS_DSC_USD };
        DEFINITIONS_UNITS[1][1] = new InstrumentDefinition<?>[][] { DEFINITIONS_FWD3_USD_2 };
        DEFINITIONS_UNITS[2][0] = new InstrumentDefinition<?>[][] { DEFINITIONS_DSC_USD };
        DEFINITIONS_UNITS[2][1] = new InstrumentDefinition<?>[][] { DEFINITIONS_FWD3_USD_2 };
        DEFINITIONS_UNITS[3][0] = new InstrumentDefinition<?>[][] { DEFINITIONS_DSC_USD };
        DEFINITIONS_UNITS[3][1] = new InstrumentDefinition<?>[][] { DEFINITIONS_FWD3_USD_2 };
        DEFINITIONS_UNITS[3][2] = new InstrumentDefinition<?>[][] { DEFINITIONS_FWD6_USD };
        DEFINITIONS_UNITS[4][0] = new InstrumentDefinition<?>[][] { DEFINITIONS_DSC_USD };
        DEFINITIONS_UNITS[4][1] = new InstrumentDefinition<?>[][] { DEFINITIONS_FWD3_USD_2 };
        DEFINITIONS_UNITS[4][2] = new InstrumentDefinition<?>[][] { DEFINITIONS_FWD6_USD };
        DEFINITIONS_UNITS[5][0] = new InstrumentDefinition<?>[][] { DEFINITIONS_FWD3_USD_3, DEFINITIONS_DSC_USD };
        DEFINITIONS_UNITS[6][0] = new InstrumentDefinition<?>[][] { DEFINITIONS_FWD3_USD_3, DEFINITIONS_DSC_USD };
        DEFINITIONS_UNITS[7][0] = new InstrumentDefinition<?>[][] { DEFINITIONS_DSC_USD };
        DEFINITIONS_UNITS[7][1] = new InstrumentDefinition<?>[][] { DEFINITIONS_FWD3_USD_4 };
        DEFINITIONS_UNITS[8][0] = new InstrumentDefinition<?>[][] { DEFINITIONS_DSC_USD };
        DEFINITIONS_UNITS[9][0] = new InstrumentDefinition<?>[][] { DEFINITIONS_DSC_USD };
        final GeneratorYDCurve genIntDQ = new GeneratorCurveYieldInterpolated(MATURITY_CALCULATOR, INTERPOLATOR_DQ);
        final GeneratorYDCurve genIntLin = new GeneratorCurveYieldInterpolated(MATURITY_CALCULATOR, INTERPOLATOR_LINEAR);
        final int compoundingRate = 1;
        final GeneratorYDCurve genIntRPLin = new GeneratorCurveYieldPeriodicInterpolated(MATURITY_CALCULATOR, compoundingRate, INTERPOLATOR_LINEAR);
        final GeneratorYDCurve genIntDFLL = new GeneratorCurveDiscountFactorInterpolated(MATURITY_CALCULATOR, INTERPOLATOR_LL);
        final GeneratorYDCurve genNS = new GeneratorCurveYieldNelsonSiegel();
        final GeneratorYDCurve genInt0 = new GeneratorCurveYieldInterpolatedAnchor(MATURITY_CALCULATOR, INTERPOLATOR_LINEAR);
        final GeneratorYDCurve genAddExistFwd3 = new GeneratorCurveAddYieldExisiting(genIntDQ, false, CURVE_NAME_FWD3_USD);
        final LocalDate startTOY = LocalDate.of(2011, 12, 30);
        final LocalDate endTOY = LocalDate.of(2012, 1, 2);
        // 25bps
        final double spreadTOY = 0.0025;
        final double dfTOY = 1.0 / (1 + DAY_COUNT_CASH.getDayCountFraction(startTOY, endTOY) * spreadTOY);
        final LocalDate startTOQ = LocalDate.of(2012, 3, 30);
        final LocalDate endTOQ = LocalDate.of(2012, 4, 2);
        // 25bps
        final double spreadTOQ = 0.0010;
        final double dfTOQ = 1.0 / (1 + DAY_COUNT_CASH.getDayCountFraction(startTOQ, endTOQ) * spreadTOQ);
        final double[] times = { TimeCalculator.getTimeBetween(NOW, startTOY), TimeCalculator.getTimeBetween(NOW, endTOY), TimeCalculator.getTimeBetween(NOW, startTOQ), TimeCalculator.getTimeBetween(NOW, endTOQ) };
        final double[] df = { 1.0, dfTOY, dfTOY, dfTOY * dfTOQ };
        final YieldAndDiscountCurve curveTOY = new DiscountCurve("TOY", new InterpolatedDoublesCurve(times, df, INTERPOLATOR_LINEAR, true));
        final GeneratorYDCurve genAddFixed = new GeneratorCurveAddYieldFixed(genIntDQ, false, curveTOY);
        final GeneratorYDCurve genIntDQ0 = new GeneratorCurveYieldInterpolatedAnchor(MATURITY_CALCULATOR, INTERPOLATOR_DQ);
        final int[] nbParameters = { 5, DSC_USD_MARKET_QUOTES.length - 5 };
        final GeneratorYDCurve gen2Blocks = new GeneratorCurveAddYieldNb(new GeneratorYDCurve[] { genIntDFLL, genIntDQ0 }, nbParameters, false);
        GENERATORS_UNITS[0][0] = new GeneratorYDCurve[] { genIntLin };
        GENERATORS_UNITS[0][1] = new GeneratorYDCurve[] { genIntLin };
        GENERATORS_UNITS[1][0] = new GeneratorYDCurve[] { genIntRPLin };
        GENERATORS_UNITS[1][1] = new GeneratorYDCurve[] { genIntRPLin };
        GENERATORS_UNITS[2][0] = new GeneratorYDCurve[] { genIntDFLL };
        GENERATORS_UNITS[2][1] = new GeneratorYDCurve[] { genIntDFLL };
        // 3xinterpolated / 2xinterpolated + spread over existing
        GENERATORS_UNITS[3][0] = new GeneratorYDCurve[] { genIntDQ };
        GENERATORS_UNITS[3][1] = new GeneratorYDCurve[] { genIntDQ };
        GENERATORS_UNITS[3][2] = new GeneratorYDCurve[] { genIntDQ };
        GENERATORS_UNITS[4][0] = new GeneratorYDCurve[] { genIntDQ };
        GENERATORS_UNITS[4][1] = new GeneratorYDCurve[] { genIntDQ };
        GENERATORS_UNITS[4][2] = new GeneratorYDCurve[] { genAddExistFwd3 };
        // 2xinterpolated / interpolated + spread over existing
        GENERATORS_UNITS[5][0] = new GeneratorYDCurve[] { genIntDQ, genIntDQ };
        GENERATORS_UNITS[6][0] = new GeneratorYDCurve[] { genIntDQ, genAddExistFwd3 };
        // interpolated + functional+interpolated
        GENERATORS_UNITS[7][0] = new GeneratorYDCurve[] { genIntDQ };
        GENERATORS_UNITS[7][1] = new GeneratorYDCurve[] { new GeneratorCurveAddYield(new GeneratorYDCurve[] { genNS, genInt0 }, false) };
        GENERATORS_UNITS[8][0] = new GeneratorYDCurve[] { genAddFixed };
        GENERATORS_UNITS[9][0] = new GeneratorYDCurve[] { gen2Blocks };
        NAMES_UNITS[0][0] = new String[] { CURVE_NAME_DSC_USD };
        NAMES_UNITS[0][1] = new String[] { CURVE_NAME_FWD3_USD };
        NAMES_UNITS[1][0] = new String[] { CURVE_NAME_DSC_USD };
        NAMES_UNITS[1][1] = new String[] { CURVE_NAME_FWD3_USD };
        NAMES_UNITS[2][0] = new String[] { CURVE_NAME_DSC_USD };
        NAMES_UNITS[2][1] = new String[] { CURVE_NAME_FWD3_USD };
        NAMES_UNITS[3][0] = new String[] { CURVE_NAME_DSC_USD };
        NAMES_UNITS[3][1] = new String[] { CURVE_NAME_FWD3_USD };
        NAMES_UNITS[3][2] = new String[] { CURVE_NAME_FWD6_USD };
        NAMES_UNITS[4][0] = new String[] { CURVE_NAME_DSC_USD };
        NAMES_UNITS[4][1] = new String[] { CURVE_NAME_FWD3_USD };
        NAMES_UNITS[4][2] = new String[] { CURVE_NAME_FWD6_USD };
        NAMES_UNITS[5][0] = new String[] { CURVE_NAME_FWD3_USD, CURVE_NAME_DSC_USD };
        NAMES_UNITS[6][0] = new String[] { CURVE_NAME_FWD3_USD, CURVE_NAME_DSC_USD };
        NAMES_UNITS[7][0] = new String[] { CURVE_NAME_DSC_USD };
        NAMES_UNITS[7][1] = new String[] { CURVE_NAME_FWD3_USD };
        NAMES_UNITS[8][0] = new String[] { CURVE_NAME_DSC_USD };
        NAMES_UNITS[9][0] = new String[] { CURVE_NAME_DSC_USD };
    }
    // Present Value
    // Par spread market quote
    // Instrument used for sensitivity tests
}
