/**
 * Copyright (C) 2012 - present by OpenGamma Inc. and the OpenGamma group of companies
 * 
 * Please see distribution for license.
 */
package com.opengamma.analytics.financial.curve;

import com.opengamma.lambdava.tuple.ObjectsPair;
import com.opengamma.lambdava.tuple.Pair;

/**
 * Build of curve in several currencies in several blocks with relevant Jacobian matrices.
 * Currencies: USD (2 curves), EUR (2 curves), JPY (3 curves)
 */
public class CurveConstructionXCcyTest {

    static {
        CCY_MAP.put(CURVE_NAME_DSC_USD, CCY_USD);
        CCY_MAP.put(CURVE_NAME_FWD3_USD, CCY_USD);
        CCY_MAP.put(CURVE_NAME_DSC_EUR, CCY_EUR);
        CCY_MAP.put(CURVE_NAME_FWD3_EUR, CCY_EUR);
        CCY_MAP.put(CURVE_NAME_DSC_JPY, CCY_JPY);
        CCY_MAP.put(CURVE_NAME_FWD3_JPY, CCY_JPY);
        CCY_MAP.put(CURVE_NAME_FWD6_JPY, CCY_JPY);
        FX_MATRIX.addCurrency(CCY_EUR, CCY_USD, FX_EURUSD);
        FX_MATRIX.addCurrency(CCY_JPY, CCY_USD, 1 / FX_USDJPY);
    }

    static {
        DEFINITIONS_DSC_USD = getDefinitions(DSC_1_MARKET_QUOTES, DSC_1_GENERATORS, DSC_1_TENOR);
        DEFINITIONS_FWD3_USD = getDefinitions(FWD_1_MARKET_QUOTES, FWD_1_GENERATORS, FWD_1_TENOR);
        DEFINITIONS_DSC_EUR = getDefinitions(DSC_EUR_MARKET_QUOTES, DSC_EUR_GENERATORS, DSC_EUR_TENOR);
        DEFINITIONS_FWD3_EUR = getDefinitions(FWD_EUR_MARKET_QUOTES, FWD_EUR_GENERATORS, FWD_EUR_TENOR);
        DEFINITIONS_DSC_JPY = getDefinitions(DSC_JPY_MARKET_QUOTES, DSC_JPY_GENERATORS, DSC_JPY_TENOR);
        DEFINITIONS_FWD3_JPY = getDefinitions(FWD3_JPY_MARKET_QUOTES, FWD3_JPY_GENERATORS, FWD3_JPY_TENOR);
        DEFINITIONS_FWD6_JPY = getDefinitions(FWD6_JPY_MARKET_QUOTES, FWD6_JPY_GENERATORS, FWD6_JPY_TENOR);
        DEFINITIONS_UNITS[0] = new InstrumentDefinition<?>[NB_UNITS[0]][][];
        DEFINITIONS_UNITS[1] = new InstrumentDefinition<?>[NB_UNITS[1]][][];
        DEFINITIONS_UNITS[2] = new InstrumentDefinition<?>[NB_UNITS[2]][][];
        DEFINITIONS_UNITS[0][0] = new InstrumentDefinition<?>[][] { DEFINITIONS_DSC_USD };
        DEFINITIONS_UNITS[0][1] = new InstrumentDefinition<?>[][] { DEFINITIONS_FWD3_USD };
        DEFINITIONS_UNITS[0][2] = new InstrumentDefinition<?>[][] { DEFINITIONS_DSC_EUR, DEFINITIONS_FWD3_EUR };
        DEFINITIONS_UNITS[1][0] = new InstrumentDefinition<?>[][] { DEFINITIONS_DSC_USD };
        DEFINITIONS_UNITS[1][1] = new InstrumentDefinition<?>[][] { DEFINITIONS_FWD3_USD };
        DEFINITIONS_UNITS[1][2] = new InstrumentDefinition<?>[][] { DEFINITIONS_DSC_JPY, DEFINITIONS_FWD3_JPY, DEFINITIONS_FWD6_JPY };
        DEFINITIONS_UNITS[2][0] = new InstrumentDefinition<?>[][] { DEFINITIONS_DSC_USD, DEFINITIONS_FWD3_USD, DEFINITIONS_DSC_JPY, DEFINITIONS_FWD3_JPY, DEFINITIONS_FWD6_JPY };
        final GeneratorYDCurve genInt = new GeneratorCurveYieldInterpolated(MATURITY_CALCULATOR, INTERPOLATOR);
        GENERATORS_UNITS[0] = new GeneratorYDCurve[NB_UNITS[0]][];
        GENERATORS_UNITS[1] = new GeneratorYDCurve[NB_UNITS[1]][];
        GENERATORS_UNITS[2] = new GeneratorYDCurve[NB_UNITS[2]][];
        GENERATORS_UNITS[0][0] = new GeneratorYDCurve[] { genInt };
        GENERATORS_UNITS[0][1] = new GeneratorYDCurve[] { genInt };
        GENERATORS_UNITS[0][2] = new GeneratorYDCurve[] { genInt, genInt };
        GENERATORS_UNITS[1][0] = new GeneratorYDCurve[] { genInt };
        GENERATORS_UNITS[1][1] = new GeneratorYDCurve[] { genInt };
        GENERATORS_UNITS[1][2] = new GeneratorYDCurve[] { genInt, genInt, genInt };
        GENERATORS_UNITS[2][0] = new GeneratorYDCurve[] { genInt, genInt, genInt, genInt, genInt };
        NAMES_UNITS[0] = new String[NB_UNITS[0]][];
        NAMES_UNITS[1] = new String[NB_UNITS[1]][];
        NAMES_UNITS[2] = new String[NB_UNITS[2]][];
        NAMES_UNITS[0][0] = new String[] { CURVE_NAME_DSC_USD };
        NAMES_UNITS[0][1] = new String[] { CURVE_NAME_FWD3_USD };
        NAMES_UNITS[0][2] = new String[] { CURVE_NAME_DSC_EUR, CURVE_NAME_FWD3_EUR };
        NAMES_UNITS[1][0] = new String[] { CURVE_NAME_DSC_USD };
        NAMES_UNITS[1][1] = new String[] { CURVE_NAME_FWD3_USD };
        NAMES_UNITS[1][2] = new String[] { CURVE_NAME_DSC_JPY, CURVE_NAME_FWD3_JPY, CURVE_NAME_FWD6_JPY };
        NAMES_UNITS[2][0] = new String[] { CURVE_NAME_DSC_USD, CURVE_NAME_FWD3_USD, CURVE_NAME_DSC_JPY, CURVE_NAME_FWD3_JPY, CURVE_NAME_FWD6_JPY };
    }
    // Present Value
    // Par spread market quote
}
